from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Project")


try:
    from ..models import issue_folder
except ImportError:
    import sys

    issue_folder = sys.modules[__package__ + "issue_folder"]


@attr.s(auto_attribs=True)
class Project(issue_folder.IssueFolder):
    """Represents a YouTrack project."""

    starting_number: "Union[Unset, int]" = UNSET
    short_name: "Union[Unset, str]" = UNSET
    description: "Union[Unset, str]" = UNSET
    leader: "Union[Unset, user_m.User]" = UNSET
    created_by: "Union[Unset, user_m.User]" = UNSET
    issues: "Union[Unset, List[issue_m.Issue]]" = UNSET
    custom_fields: "Union[Unset, project_custom_fields_m.ProjectCustomFields]" = UNSET
    archived: "Union[Unset, bool]" = UNSET
    from_email: "Union[Unset, str]" = UNSET
    reply_to_email: "Union[Unset, str]" = UNSET
    template: "Union[Unset, bool]" = UNSET
    icon_url: "Union[Unset, str]" = UNSET
    team: "Union[Unset, user_group_m.UserGroup]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        starting_number = self.starting_number
        short_name = self.short_name
        description = self.description
        leader: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.leader, Unset):
            leader = self.leader.to_dict()

        created_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        issues: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.issues, Unset):
            issues = []
            for issues_item_data in self.issues:
                issues_item = issues_item_data.to_dict()

                issues.append(issues_item)

        custom_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_fields, Unset):
            custom_fields = self.custom_fields.to_dict()

        archived = self.archived
        from_email = self.from_email
        reply_to_email = self.reply_to_email
        template = self.template
        icon_url = self.icon_url
        team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.team, Unset):
            team = self.team.to_dict()

        field_dict: Dict[str, Any] = {}
        _IssueFolder_dict = super().to_dict()
        field_dict.update(_IssueFolder_dict)
        field_dict.update({})
        if starting_number is not UNSET:
            field_dict["startingNumber"] = starting_number
        if short_name is not UNSET:
            field_dict["shortName"] = short_name
        if description is not UNSET:
            field_dict["description"] = description
        if leader is not UNSET:
            field_dict["leader"] = leader
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if issues is not UNSET:
            field_dict["issues"] = issues
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if archived is not UNSET:
            field_dict["archived"] = archived
        if from_email is not UNSET:
            field_dict["fromEmail"] = from_email
        if reply_to_email is not UNSET:
            field_dict["replyToEmail"] = reply_to_email
        if template is not UNSET:
            field_dict["template"] = template
        if icon_url is not UNSET:
            field_dict["iconUrl"] = icon_url
        if team is not UNSET:
            field_dict["team"] = team

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
            from ..models import project_custom_fields as project_custom_fields_m
            from ..models import user as user_m
            from ..models import user_group as user_group_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            user_group_m = sys.modules[__package__ + "user_group"]
            user_m = sys.modules[__package__ + "user"]
            project_custom_fields_m = sys.modules[__package__ + "project_custom_fields"]

        d = src_dict.copy()

        _IssueFolder_kwargs = super().from_dict(src_dict=d).to_dict()
        _IssueFolder_kwargs.pop("$type")

        starting_number = d.pop("startingNumber", UNSET)

        short_name = d.pop("shortName", UNSET)

        description = d.pop("description", UNSET)

        _leader = d.pop("leader", UNSET)
        leader: Union[Unset, user_m.User]
        if isinstance(_leader, Unset):
            leader = UNSET
        else:
            leader = user_m.User.from_dict(_leader)

        _created_by = d.pop("createdBy", UNSET)
        created_by: Union[Unset, user_m.User]
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = user_m.User.from_dict(_created_by)

        issues = []
        _issues = d.pop("issues", UNSET)
        for issues_item_data in _issues or []:
            issues_item = issue_m.Issue.from_dict(issues_item_data)

            issues.append(issues_item)

        _custom_fields = d.pop("customFields", UNSET)
        custom_fields: Union[Unset, project_custom_fields_m.ProjectCustomFields]
        if isinstance(_custom_fields, Unset):
            custom_fields = UNSET
        else:
            custom_fields = project_custom_fields_m.ProjectCustomFields.from_dict(_custom_fields)

        archived = d.pop("archived", UNSET)

        from_email = d.pop("fromEmail", UNSET)

        reply_to_email = d.pop("replyToEmail", UNSET)

        template = d.pop("template", UNSET)

        icon_url = d.pop("iconUrl", UNSET)

        _team = d.pop("team", UNSET)
        team: Union[Unset, user_group_m.UserGroup]
        if isinstance(_team, Unset):
            team = UNSET
        else:
            team = user_group_m.UserGroup.from_dict(_team)

        project = cls(
            starting_number=starting_number,
            short_name=short_name,
            description=description,
            leader=leader,
            created_by=created_by,
            issues=issues,
            custom_fields=custom_fields,
            archived=archived,
            from_email=from_email,
            reply_to_email=reply_to_email,
            template=template,
            icon_url=icon_url,
            team=team,
            **_IssueFolder_kwargs,
        )

        return project



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomFilterField")


try:
    from ..models import filter_field
except ImportError:
    import sys

    filter_field = sys.modules[__package__ + "filter_field"]


@attr.s(auto_attribs=True)
class CustomFilterField(filter_field.FilterField):
    """Represents a custom field of the issue."""

    custom_field: "Union[Unset, custom_field_m.CustomField]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        custom_field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_field, Unset):
            custom_field = self.custom_field.to_dict()

        field_dict: Dict[str, Any] = {}
        _FilterField_dict = super().to_dict()
        field_dict.update(_FilterField_dict)
        field_dict.update({})
        if custom_field is not UNSET:
            field_dict["customField"] = custom_field

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import custom_field as custom_field_m
        except ImportError:
            import sys

            custom_field_m = sys.modules[__package__ + "custom_field"]

        d = src_dict.copy()

        _FilterField_kwargs = super().from_dict(src_dict=d).to_dict()
        _FilterField_kwargs.pop("$type")

        _custom_field = d.pop("customField", UNSET)
        custom_field: Union[Unset, custom_field_m.CustomField]
        if isinstance(_custom_field, Unset):
            custom_field = UNSET
        else:
            custom_field = custom_field_m.CustomField.from_dict(_custom_field)

        custom_filter_field = cls(
            custom_field=custom_field,
            **_FilterField_kwargs,
        )

        return custom_filter_field



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models import enum_bundle as enum_bundle_m
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: "str",
    *,
    client: Client,
    json_body: enum_bundle_m.EnumBundle,
    fields: "Union[Unset, None, str]" = "$type,id",
) -> Dict[str, Any]:
    url = "{}/admin/customFieldSettings/bundles/enum/{id}".format(client.youtrack_base_url, id=id)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "fields": fields,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[enum_bundle_m.EnumBundle]:
    if response.status_code == 200:
        response_200 = enum_bundle_m.EnumBundle.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[enum_bundle_m.EnumBundle]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: "str",
    *,
    client: Client,
    json_body: enum_bundle_m.EnumBundle,
    fields: "Union[Unset, None, str]" = "$type,id",
) -> Response[enum_bundle_m.EnumBundle]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
        json_body=json_body,
        fields=fields,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: "str",
    *,
    client: Client,
    json_body: enum_bundle_m.EnumBundle,
    fields: "Union[Unset, None, str]" = "$type,id",
) -> Optional[enum_bundle_m.EnumBundle]:
    """ """

    return sync_detailed(
        id=id,
        client=client,
        json_body=json_body,
        fields=fields,
    ).parsed


async def asyncio_detailed(
    id: "str",
    *,
    client: Client,
    json_body: enum_bundle_m.EnumBundle,
    fields: "Union[Unset, None, str]" = "$type,id",
) -> Response[enum_bundle_m.EnumBundle]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
        json_body=json_body,
        fields=fields,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: "str",
    *,
    client: Client,
    json_body: enum_bundle_m.EnumBundle,
    fields: "Union[Unset, None, str]" = "$type,id",
) -> Optional[enum_bundle_m.EnumBundle]:
    """ """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            json_body=json_body,
            fields=fields,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TextCustomFieldActivityItem")


try:
    from ..models import custom_field_activity_item
except ImportError:
    import sys

    custom_field_activity_item = sys.modules[__package__ + "custom_field_activity_item"]


@attr.s(auto_attribs=True)
class TextCustomFieldActivityItem(custom_field_activity_item.CustomFieldActivityItem):
    """Represents an activity that affects a custom field of the `text` type of an issue."""

    target: "Union[Unset, issue_m.Issue]" = UNSET
    removed: "Union[Unset, str]" = UNSET
    added: "Union[Unset, str]" = UNSET
    markup: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        target: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.target, Unset):
            target = self.target.to_dict()

        removed = self.removed
        added = self.added
        markup = self.markup

        field_dict: Dict[str, Any] = {}
        _CustomFieldActivityItem_dict = super().to_dict()
        field_dict.update(_CustomFieldActivityItem_dict)
        field_dict.update({})
        if target is not UNSET:
            field_dict["target"] = target
        if removed is not UNSET:
            field_dict["removed"] = removed
        if added is not UNSET:
            field_dict["added"] = added
        if markup is not UNSET:
            field_dict["markup"] = markup

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]

        d = src_dict.copy()

        _CustomFieldActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _CustomFieldActivityItem_kwargs.pop("$type")

        _target = d.pop("target", UNSET)
        target: Union[Unset, issue_m.Issue]
        if isinstance(_target, Unset):
            target = UNSET
        else:
            target = issue_m.Issue.from_dict(_target)

        removed = d.pop("removed", UNSET)

        added = d.pop("added", UNSET)

        markup = d.pop("markup", UNSET)

        text_custom_field_activity_item = cls(
            target=target,
            removed=removed,
            added=added,
            markup=markup,
            **_CustomFieldActivityItem_kwargs,
        )

        return text_custom_field_activity_item



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="MultiValueActivityItem")


try:
    from ..models import activity_item
except ImportError:
    import sys

    activity_item = sys.modules[__package__ + "activity_item"]


@attr.s(auto_attribs=True)
class MultiValueActivityItem(activity_item.ActivityItem):
    """Represents a change in an entity attribute that has type of `Array of &lt;Entities&gt;`."""

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        _ActivityItem_dict = super().to_dict()
        field_dict.update(_ActivityItem_dict)
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _ActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _ActivityItem_kwargs.pop("$type")

        multi_value_activity_item = cls(
            **_ActivityItem_kwargs,
        )

        return multi_value_activity_item



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueWatchers")


@attr.s(auto_attribs=True)
class IssueWatchers:
    """Represents users that are subscribed to notifications about the issue."""

    has_star: "Union[Unset, bool]" = UNSET
    issue_watchers: "Union[Unset, List[issue_watcher_m.IssueWatcher]]" = UNSET
    duplicate_watchers: "Union[Unset, List[issue_watcher_m.IssueWatcher]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        has_star = self.has_star
        issue_watchers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.issue_watchers, Unset):
            issue_watchers = []
            for issue_watchers_item_data in self.issue_watchers:
                issue_watchers_item = issue_watchers_item_data.to_dict()

                issue_watchers.append(issue_watchers_item)

        duplicate_watchers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.duplicate_watchers, Unset):
            duplicate_watchers = []
            for duplicate_watchers_item_data in self.duplicate_watchers:
                duplicate_watchers_item = duplicate_watchers_item_data.to_dict()

                duplicate_watchers.append(duplicate_watchers_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if has_star is not UNSET:
            field_dict["hasStar"] = has_star
        if issue_watchers is not UNSET:
            field_dict["issueWatchers"] = issue_watchers
        if duplicate_watchers is not UNSET:
            field_dict["duplicateWatchers"] = duplicate_watchers
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue_watcher as issue_watcher_m
        except ImportError:
            import sys

            issue_watcher_m = sys.modules[__package__ + "issue_watcher"]

        d = src_dict.copy()

        has_star = d.pop("hasStar", UNSET)

        issue_watchers = []
        _issue_watchers = d.pop("issueWatchers", UNSET)
        for issue_watchers_item_data in _issue_watchers or []:
            issue_watchers_item = issue_watcher_m.IssueWatcher.from_dict(issue_watchers_item_data)

            issue_watchers.append(issue_watchers_item)

        duplicate_watchers = []
        _duplicate_watchers = d.pop("duplicateWatchers", UNSET)
        for duplicate_watchers_item_data in _duplicate_watchers or []:
            duplicate_watchers_item = issue_watcher_m.IssueWatcher.from_dict(duplicate_watchers_item_data)

            duplicate_watchers.append(duplicate_watchers_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue_watchers = cls(
            has_star=has_star,
            issue_watchers=issue_watchers,
            duplicate_watchers=duplicate_watchers,
            id=id,
            type=type,
        )

        return issue_watchers



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="WIPLimit")


@attr.s(auto_attribs=True)
class WIPLimit:
    """Represents WIP limits for particular column. If they are not satisfied, the column will be highlighted in UI."""

    max_: "Union[Unset, int]" = UNSET
    min_: "Union[Unset, int]" = UNSET
    column: "Union[Unset, agile_column_m.AgileColumn]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        max_ = self.max_
        min_ = self.min_
        column: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.column, Unset):
            column = self.column.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if max_ is not UNSET:
            field_dict["max"] = max_
        if min_ is not UNSET:
            field_dict["min"] = min_
        if column is not UNSET:
            field_dict["column"] = column
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import agile_column as agile_column_m
        except ImportError:
            import sys

            agile_column_m = sys.modules[__package__ + "agile_column"]

        d = src_dict.copy()

        max_ = d.pop("max", UNSET)

        min_ = d.pop("min", UNSET)

        _column = d.pop("column", UNSET)
        column: Union[Unset, agile_column_m.AgileColumn]
        if isinstance(_column, Unset):
            column = UNSET
        else:
            column = agile_column_m.AgileColumn.from_dict(_column)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        wip_limit = cls(
            max_=max_,
            min_=min_,
            column=column,
            id=id,
            type=type,
        )

        return wip_limit



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="User")


@attr.s(auto_attribs=True)
class User:
    """Represents a user in YouTrack. Please note that the read-only properties of a user account, like
    credentials, or email and so on, you can only change in
    <a href="https://www.jetbrains.com/help/youtrack/devportal/?Hub-REST-API">Hub REST API</a>."""

    login: "Union[Unset, str]" = UNSET
    full_name: "Union[Unset, str]" = UNSET
    email: "Union[Unset, str]" = UNSET
    jabber_account_name: "Union[Unset, str]" = UNSET
    ring_id: "Union[Unset, str]" = UNSET
    guest: "Union[Unset, bool]" = UNSET
    online: "Union[Unset, bool]" = UNSET
    banned: "Union[Unset, bool]" = UNSET
    tags: "Union[Unset, List[issue_tag_m.IssueTag]]" = UNSET
    saved_queries: "Union[Unset, List[saved_query_m.SavedQuery]]" = UNSET
    avatar_url: "Union[Unset, str]" = UNSET
    profiles: "Union[Unset, user_profiles_m.UserProfiles]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        login = self.login
        full_name = self.full_name
        email = self.email
        jabber_account_name = self.jabber_account_name
        ring_id = self.ring_id
        guest = self.guest
        online = self.online
        banned = self.banned
        tags: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = []
            for tags_item_data in self.tags:
                tags_item = tags_item_data.to_dict()

                tags.append(tags_item)

        saved_queries: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.saved_queries, Unset):
            saved_queries = []
            for saved_queries_item_data in self.saved_queries:
                saved_queries_item = saved_queries_item_data.to_dict()

                saved_queries.append(saved_queries_item)

        avatar_url = self.avatar_url
        profiles: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profiles, Unset):
            profiles = self.profiles.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if login is not UNSET:
            field_dict["login"] = login
        if full_name is not UNSET:
            field_dict["fullName"] = full_name
        if email is not UNSET:
            field_dict["email"] = email
        if jabber_account_name is not UNSET:
            field_dict["jabberAccountName"] = jabber_account_name
        if ring_id is not UNSET:
            field_dict["ringId"] = ring_id
        if guest is not UNSET:
            field_dict["guest"] = guest
        if online is not UNSET:
            field_dict["online"] = online
        if banned is not UNSET:
            field_dict["banned"] = banned
        if tags is not UNSET:
            field_dict["tags"] = tags
        if saved_queries is not UNSET:
            field_dict["savedQueries"] = saved_queries
        if avatar_url is not UNSET:
            field_dict["avatarUrl"] = avatar_url
        if profiles is not UNSET:
            field_dict["profiles"] = profiles
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue_tag as issue_tag_m
            from ..models import saved_query as saved_query_m
            from ..models import user_profiles as user_profiles_m
        except ImportError:
            import sys

            user_profiles_m = sys.modules[__package__ + "user_profiles"]
            saved_query_m = sys.modules[__package__ + "saved_query"]
            issue_tag_m = sys.modules[__package__ + "issue_tag"]

        d = src_dict.copy()

        login = d.pop("login", UNSET)

        full_name = d.pop("fullName", UNSET)

        email = d.pop("email", UNSET)

        jabber_account_name = d.pop("jabberAccountName", UNSET)

        ring_id = d.pop("ringId", UNSET)

        guest = d.pop("guest", UNSET)

        online = d.pop("online", UNSET)

        banned = d.pop("banned", UNSET)

        tags = []
        _tags = d.pop("tags", UNSET)
        for tags_item_data in _tags or []:
            tags_item = issue_tag_m.IssueTag.from_dict(tags_item_data)

            tags.append(tags_item)

        saved_queries = []
        _saved_queries = d.pop("savedQueries", UNSET)
        for saved_queries_item_data in _saved_queries or []:
            saved_queries_item = saved_query_m.SavedQuery.from_dict(saved_queries_item_data)

            saved_queries.append(saved_queries_item)

        avatar_url = d.pop("avatarUrl", UNSET)

        _profiles = d.pop("profiles", UNSET)
        profiles: Union[Unset, user_profiles_m.UserProfiles]
        if isinstance(_profiles, Unset):
            profiles = UNSET
        else:
            profiles = user_profiles_m.UserProfiles.from_dict(_profiles)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        user = cls(
            login=login,
            full_name=full_name,
            email=email,
            jabber_account_name=jabber_account_name,
            ring_id=ring_id,
            guest=guest,
            online=online,
            banned=banned,
            tags=tags,
            saved_queries=saved_queries,
            avatar_url=avatar_url,
            profiles=profiles,
            id=id,
            type=type,
        )

        return user



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UsesMarkupActivityItem")


try:
    from ..models import simple_value_activity_item
except ImportError:
    import sys

    simple_value_activity_item = sys.modules[__package__ + "simple_value_activity_item"]


@attr.s(auto_attribs=True)
class UsesMarkupActivityItem(simple_value_activity_item.SimpleValueActivityItem):
    """Represents the change of the boolean flag that indicates whether YouTrack Wiki or Markdown
    is used in the target entity as a markup language. If "true", then the markdown is used.
    Otherwise, YouTrack Wiki markup."""

    removed: "Union[Unset, bool]" = UNSET
    added: "Union[Unset, bool]" = UNSET
    markup: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        removed = self.removed
        added = self.added
        markup = self.markup

        field_dict: Dict[str, Any] = {}
        _SimpleValueActivityItem_dict = super().to_dict()
        field_dict.update(_SimpleValueActivityItem_dict)
        field_dict.update({})
        if removed is not UNSET:
            field_dict["removed"] = removed
        if added is not UNSET:
            field_dict["added"] = added
        if markup is not UNSET:
            field_dict["markup"] = markup

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _SimpleValueActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _SimpleValueActivityItem_kwargs.pop("$type")

        removed = d.pop("removed", UNSET)

        added = d.pop("added", UNSET)

        markup = d.pop("markup", UNSET)

        uses_markup_activity_item = cls(
            removed=removed,
            added=added,
            markup=markup,
            **_SimpleValueActivityItem_kwargs,
        )

        return uses_markup_activity_item



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Suggestion")


@attr.s(auto_attribs=True)
class Suggestion:
    """Represents query suggestion."""

    completion_start: "Union[Unset, int]" = UNSET
    completion_end: "Union[Unset, int]" = UNSET
    matching_start: "Union[Unset, int]" = UNSET
    matching_end: "Union[Unset, int]" = UNSET
    caret: "Union[Unset, int]" = UNSET
    description: "Union[Unset, str]" = UNSET
    option: "Union[Unset, str]" = UNSET
    prefix: "Union[Unset, str]" = UNSET
    suffix: "Union[Unset, str]" = UNSET
    group: "Union[Unset, str]" = UNSET
    icon: "Union[Unset, str]" = UNSET
    auxiliary_icon: "Union[Unset, str]" = UNSET
    class_name: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        completion_start = self.completion_start
        completion_end = self.completion_end
        matching_start = self.matching_start
        matching_end = self.matching_end
        caret = self.caret
        description = self.description
        option = self.option
        prefix = self.prefix
        suffix = self.suffix
        group = self.group
        icon = self.icon
        auxiliary_icon = self.auxiliary_icon
        class_name = self.class_name
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if completion_start is not UNSET:
            field_dict["completionStart"] = completion_start
        if completion_end is not UNSET:
            field_dict["completionEnd"] = completion_end
        if matching_start is not UNSET:
            field_dict["matchingStart"] = matching_start
        if matching_end is not UNSET:
            field_dict["matchingEnd"] = matching_end
        if caret is not UNSET:
            field_dict["caret"] = caret
        if description is not UNSET:
            field_dict["description"] = description
        if option is not UNSET:
            field_dict["option"] = option
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if suffix is not UNSET:
            field_dict["suffix"] = suffix
        if group is not UNSET:
            field_dict["group"] = group
        if icon is not UNSET:
            field_dict["icon"] = icon
        if auxiliary_icon is not UNSET:
            field_dict["auxiliaryIcon"] = auxiliary_icon
        if class_name is not UNSET:
            field_dict["className"] = class_name
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        completion_start = d.pop("completionStart", UNSET)

        completion_end = d.pop("completionEnd", UNSET)

        matching_start = d.pop("matchingStart", UNSET)

        matching_end = d.pop("matchingEnd", UNSET)

        caret = d.pop("caret", UNSET)

        description = d.pop("description", UNSET)

        option = d.pop("option", UNSET)

        prefix = d.pop("prefix", UNSET)

        suffix = d.pop("suffix", UNSET)

        group = d.pop("group", UNSET)

        icon = d.pop("icon", UNSET)

        auxiliary_icon = d.pop("auxiliaryIcon", UNSET)

        class_name = d.pop("className", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        suggestion = cls(
            completion_start=completion_start,
            completion_end=completion_end,
            matching_start=matching_start,
            matching_end=matching_end,
            caret=caret,
            description=description,
            option=option,
            prefix=prefix,
            suffix=suffix,
            group=group,
            icon=icon,
            auxiliary_icon=auxiliary_icon,
            class_name=class_name,
            id=id,
            type=type,
        )

        return suggestion



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Sprint")


@attr.s(auto_attribs=True)
class Sprint:
    """Represents a sprint that is associated with an agile board. Each sprint can include issues from one or more projects."""

    agile: "Union[Unset, agile_m.Agile]" = UNSET
    name: "Union[Unset, str]" = UNSET
    goal: "Union[Unset, str]" = UNSET
    start: "Union[Unset, int]" = UNSET
    finish: "Union[Unset, int]" = UNSET
    archived: "Union[Unset, bool]" = UNSET
    is_default: "Union[Unset, bool]" = UNSET
    issues: "Union[Unset, List[issue_m.Issue]]" = UNSET
    unresolved_issues_count: "Union[Unset, int]" = UNSET
    previous_sprint: "Union[Unset, sprint_m.Sprint]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        agile: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.agile, Unset):
            agile = self.agile.to_dict()

        name = self.name
        goal = self.goal
        start = self.start
        finish = self.finish
        archived = self.archived
        is_default = self.is_default
        issues: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.issues, Unset):
            issues = []
            for issues_item_data in self.issues:
                issues_item = issues_item_data.to_dict()

                issues.append(issues_item)

        unresolved_issues_count = self.unresolved_issues_count
        previous_sprint: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.previous_sprint, Unset):
            previous_sprint = self.previous_sprint.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if agile is not UNSET:
            field_dict["agile"] = agile
        if name is not UNSET:
            field_dict["name"] = name
        if goal is not UNSET:
            field_dict["goal"] = goal
        if start is not UNSET:
            field_dict["start"] = start
        if finish is not UNSET:
            field_dict["finish"] = finish
        if archived is not UNSET:
            field_dict["archived"] = archived
        if is_default is not UNSET:
            field_dict["isDefault"] = is_default
        if issues is not UNSET:
            field_dict["issues"] = issues
        if unresolved_issues_count is not UNSET:
            field_dict["unresolvedIssuesCount"] = unresolved_issues_count
        if previous_sprint is not UNSET:
            field_dict["previousSprint"] = previous_sprint
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import agile as agile_m
            from ..models import issue as issue_m
            from ..models import sprint as sprint_m
        except ImportError:
            import sys

            agile_m = sys.modules[__package__ + "agile"]
            issue_m = sys.modules[__package__ + "issue"]
            sprint_m = sys.modules[__package__ + "sprint"]

        d = src_dict.copy()

        _agile = d.pop("agile", UNSET)
        agile: Union[Unset, agile_m.Agile]
        if isinstance(_agile, Unset):
            agile = UNSET
        else:
            agile = agile_m.Agile.from_dict(_agile)

        name = d.pop("name", UNSET)

        goal = d.pop("goal", UNSET)

        start = d.pop("start", UNSET)

        finish = d.pop("finish", UNSET)

        archived = d.pop("archived", UNSET)

        is_default = d.pop("isDefault", UNSET)

        issues = []
        _issues = d.pop("issues", UNSET)
        for issues_item_data in _issues or []:
            issues_item = issue_m.Issue.from_dict(issues_item_data)

            issues.append(issues_item)

        unresolved_issues_count = d.pop("unresolvedIssuesCount", UNSET)

        _previous_sprint = d.pop("previousSprint", UNSET)
        previous_sprint: Union[Unset, sprint_m.Sprint]
        if isinstance(_previous_sprint, Unset):
            previous_sprint = UNSET
        else:
            previous_sprint = sprint_m.Sprint.from_dict(_previous_sprint)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        sprint = cls(
            agile=agile,
            name=name,
            goal=goal,
            start=start,
            finish=finish,
            archived=archived,
            is_default=is_default,
            issues=issues,
            unresolved_issues_count=unresolved_issues_count,
            previous_sprint=previous_sprint,
            id=id,
            type=type,
        )

        return sprint



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="LimitedVisibility")


try:
    from ..models import visibility
except ImportError:
    import sys

    visibility = sys.modules[__package__ + "visibility"]


@attr.s(auto_attribs=True)
class LimitedVisibility(visibility.Visibility):
    """Represents visibility limited to several users and/or groups."""

    permitted_groups: "Union[Unset, List[user_group_m.UserGroup]]" = UNSET
    permitted_users: "Union[Unset, List[user_m.User]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        permitted_groups: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.permitted_groups, Unset):
            permitted_groups = []
            for permitted_groups_item_data in self.permitted_groups:
                permitted_groups_item = permitted_groups_item_data.to_dict()

                permitted_groups.append(permitted_groups_item)

        permitted_users: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.permitted_users, Unset):
            permitted_users = []
            for permitted_users_item_data in self.permitted_users:
                permitted_users_item = permitted_users_item_data.to_dict()

                permitted_users.append(permitted_users_item)

        field_dict: Dict[str, Any] = {}
        _Visibility_dict = super().to_dict()
        field_dict.update(_Visibility_dict)
        field_dict.update({})
        if permitted_groups is not UNSET:
            field_dict["permittedGroups"] = permitted_groups
        if permitted_users is not UNSET:
            field_dict["permittedUsers"] = permitted_users

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import user as user_m
            from ..models import user_group as user_group_m
        except ImportError:
            import sys

            user_group_m = sys.modules[__package__ + "user_group"]
            user_m = sys.modules[__package__ + "user"]

        d = src_dict.copy()

        _Visibility_kwargs = super().from_dict(src_dict=d).to_dict()
        _Visibility_kwargs.pop("$type")

        permitted_groups = []
        _permitted_groups = d.pop("permittedGroups", UNSET)
        for permitted_groups_item_data in _permitted_groups or []:
            permitted_groups_item = user_group_m.UserGroup.from_dict(permitted_groups_item_data)

            permitted_groups.append(permitted_groups_item)

        permitted_users = []
        _permitted_users = d.pop("permittedUsers", UNSET)
        for permitted_users_item_data in _permitted_users or []:
            permitted_users_item = user_m.User.from_dict(permitted_users_item_data)

            permitted_users.append(permitted_users_item)

        limited_visibility = cls(
            permitted_groups=permitted_groups,
            permitted_users=permitted_users,
            **_Visibility_kwargs,
        )

        return limited_visibility



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="PredefinedFilterField")


try:
    from ..models import filter_field
except ImportError:
    import sys

    filter_field = sys.modules[__package__ + "filter_field"]


@attr.s(auto_attribs=True)
class PredefinedFilterField(filter_field.FilterField):
    """Represents a predefined field of the issue. Predefined fields are always present in an issue and
    |cannot be customized in a project. For example, `project`, `created`,
    |`updated`, `tags`, and so on."""

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        _FilterField_dict = super().to_dict()
        field_dict.update(_FilterField_dict)
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _FilterField_kwargs = super().from_dict(src_dict=d).to_dict()
        _FilterField_kwargs.pop("$type")

        predefined_filter_field = cls(
            **_FilterField_kwargs,
        )

        return predefined_filter_field



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="StateBundle")


try:
    from ..models import base_bundle
except ImportError:
    import sys

    base_bundle = sys.modules[__package__ + "base_bundle"]


@attr.s(auto_attribs=True)
class StateBundle(base_bundle.BaseBundle):
    """Represents a set of states in YouTrack."""

    values: "Union[Unset, List[state_bundle_element_m.StateBundleElement]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.values, Unset):
            values = []
            for values_item_data in self.values:
                values_item = values_item_data.to_dict()

                values.append(values_item)

        field_dict: Dict[str, Any] = {}
        _BaseBundle_dict = super().to_dict()
        field_dict.update(_BaseBundle_dict)
        field_dict.update({})
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import state_bundle_element as state_bundle_element_m
        except ImportError:
            import sys

            state_bundle_element_m = sys.modules[__package__ + "state_bundle_element"]

        d = src_dict.copy()

        _BaseBundle_kwargs = super().from_dict(src_dict=d).to_dict()
        _BaseBundle_kwargs.pop("$type")

        values = []
        _values = d.pop("values", UNSET)
        for values_item_data in _values or []:
            values_item = state_bundle_element_m.StateBundleElement.from_dict(values_item_data)

            values.append(values_item)

        state_bundle = cls(
            values=values,
            **_BaseBundle_kwargs,
        )

        return state_bundle



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models import appearance_settings as appearance_settings_m
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    json_body: appearance_settings_m.AppearanceSettings,
    fields: "Union[Unset, None, str]" = "$type,dateFieldFormat($type,datePattern,id,pattern,presentation),id,timeZone($type,id,offset,presentation)",
) -> Dict[str, Any]:
    url = "{}/admin/globalSettings/appearanceSettings".format(client.youtrack_base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "fields": fields,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[appearance_settings_m.AppearanceSettings]:
    if response.status_code == 200:
        response_200 = appearance_settings_m.AppearanceSettings.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[appearance_settings_m.AppearanceSettings]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: appearance_settings_m.AppearanceSettings,
    fields: "Union[Unset, None, str]" = "$type,dateFieldFormat($type,datePattern,id,pattern,presentation),id,timeZone($type,id,offset,presentation)",
) -> Response[appearance_settings_m.AppearanceSettings]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        fields=fields,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: appearance_settings_m.AppearanceSettings,
    fields: "Union[Unset, None, str]" = "$type,dateFieldFormat($type,datePattern,id,pattern,presentation),id,timeZone($type,id,offset,presentation)",
) -> Optional[appearance_settings_m.AppearanceSettings]:
    """ """

    return sync_detailed(
        client=client,
        json_body=json_body,
        fields=fields,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: appearance_settings_m.AppearanceSettings,
    fields: "Union[Unset, None, str]" = "$type,dateFieldFormat($type,datePattern,id,pattern,presentation),id,timeZone($type,id,offset,presentation)",
) -> Response[appearance_settings_m.AppearanceSettings]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        fields=fields,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: appearance_settings_m.AppearanceSettings,
    fields: "Union[Unset, None, str]" = "$type,dateFieldFormat($type,datePattern,id,pattern,presentation),id,timeZone($type,id,offset,presentation)",
) -> Optional[appearance_settings_m.AppearanceSettings]:
    """ """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            fields=fields,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="LocaleSettings")


@attr.s(auto_attribs=True)
class LocaleSettings:
    """Represents the System Language settings."""

    locale: "Union[Unset, locale_descriptor_m.LocaleDescriptor]" = UNSET
    is_rtl: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        locale: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.locale, Unset):
            locale = self.locale.to_dict()

        is_rtl = self.is_rtl
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if locale is not UNSET:
            field_dict["locale"] = locale
        if is_rtl is not UNSET:
            field_dict["isRTL"] = is_rtl
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import locale_descriptor as locale_descriptor_m
        except ImportError:
            import sys

            locale_descriptor_m = sys.modules[__package__ + "locale_descriptor"]

        d = src_dict.copy()

        _locale = d.pop("locale", UNSET)
        locale: Union[Unset, locale_descriptor_m.LocaleDescriptor]
        if isinstance(_locale, Unset):
            locale = UNSET
        else:
            locale = locale_descriptor_m.LocaleDescriptor.from_dict(_locale)

        is_rtl = d.pop("isRTL", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        locale_settings = cls(
            locale=locale,
            is_rtl=is_rtl,
            id=id,
            type=type,
        )

        return locale_settings



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueVoters")


@attr.s(auto_attribs=True)
class IssueVoters:
    """Represents users that have voted for the issue or its duplicates."""

    has_vote: "Union[Unset, bool]" = UNSET
    original: "Union[Unset, List[user_m.User]]" = UNSET
    duplicate: "Union[Unset, List[duplicate_vote_m.DuplicateVote]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        has_vote = self.has_vote
        original: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.original, Unset):
            original = []
            for original_item_data in self.original:
                original_item = original_item_data.to_dict()

                original.append(original_item)

        duplicate: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.duplicate, Unset):
            duplicate = []
            for duplicate_item_data in self.duplicate:
                duplicate_item = duplicate_item_data.to_dict()

                duplicate.append(duplicate_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if has_vote is not UNSET:
            field_dict["hasVote"] = has_vote
        if original is not UNSET:
            field_dict["original"] = original
        if duplicate is not UNSET:
            field_dict["duplicate"] = duplicate
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import duplicate_vote as duplicate_vote_m
            from ..models import user as user_m
        except ImportError:
            import sys

            user_m = sys.modules[__package__ + "user"]
            duplicate_vote_m = sys.modules[__package__ + "duplicate_vote"]

        d = src_dict.copy()

        has_vote = d.pop("hasVote", UNSET)

        original = []
        _original = d.pop("original", UNSET)
        for original_item_data in _original or []:
            original_item = user_m.User.from_dict(original_item_data)

            original.append(original_item)

        duplicate = []
        _duplicate = d.pop("duplicate", UNSET)
        for duplicate_item_data in _duplicate or []:
            duplicate_item = duplicate_vote_m.DuplicateVote.from_dict(duplicate_item_data)

            duplicate.append(duplicate_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue_voters = cls(
            has_vote=has_vote,
            original=original,
            duplicate=duplicate,
            id=id,
            type=type,
        )

        return issue_voters



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Telemetry")


@attr.s(auto_attribs=True)
class Telemetry:
    """Telemetry data of the YouTrack installation."""

    installation_folder: "Union[Unset, str]" = UNSET
    database_location: "Union[Unset, str]" = UNSET
    logs_location: "Union[Unset, str]" = UNSET
    available_processors: "Union[Unset, int]" = UNSET
    available_memory: "Union[Unset, str]" = UNSET
    allocated_memory: "Union[Unset, str]" = UNSET
    used_memory: "Union[Unset, str]" = UNSET
    uptime: "Union[Unset, str]" = UNSET
    started_time: "Union[Unset, int]" = UNSET
    database_background_threads: "Union[Unset, int]" = UNSET
    pending_async_jobs: "Union[Unset, int]" = UNSET
    cached_results_count_in_db_queries_cache: "Union[Unset, int]" = UNSET
    database_queries_cache_hit_rate: "Union[Unset, str]" = UNSET
    blob_strings_cache_hit_rate: "Union[Unset, str]" = UNSET
    total_transactions: "Union[Unset, int]" = UNSET
    transactions_per_second: "Union[Unset, str]" = UNSET
    requests_per_second: "Union[Unset, str]" = UNSET
    database_size: "Union[Unset, str]" = UNSET
    full_database_size: "Union[Unset, str]" = UNSET
    text_index_size: "Union[Unset, str]" = UNSET
    online_users: "Union[Unset, online_users_m.OnlineUsers]" = UNSET
    report_calculator_threads: "Union[Unset, int]" = UNSET
    notification_analyzer_threads: "Union[Unset, int]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        installation_folder = self.installation_folder
        database_location = self.database_location
        logs_location = self.logs_location
        available_processors = self.available_processors
        available_memory = self.available_memory
        allocated_memory = self.allocated_memory
        used_memory = self.used_memory
        uptime = self.uptime
        started_time = self.started_time
        database_background_threads = self.database_background_threads
        pending_async_jobs = self.pending_async_jobs
        cached_results_count_in_db_queries_cache = self.cached_results_count_in_db_queries_cache
        database_queries_cache_hit_rate = self.database_queries_cache_hit_rate
        blob_strings_cache_hit_rate = self.blob_strings_cache_hit_rate
        total_transactions = self.total_transactions
        transactions_per_second = self.transactions_per_second
        requests_per_second = self.requests_per_second
        database_size = self.database_size
        full_database_size = self.full_database_size
        text_index_size = self.text_index_size
        online_users: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.online_users, Unset):
            online_users = self.online_users.to_dict()

        report_calculator_threads = self.report_calculator_threads
        notification_analyzer_threads = self.notification_analyzer_threads
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if installation_folder is not UNSET:
            field_dict["installationFolder"] = installation_folder
        if database_location is not UNSET:
            field_dict["databaseLocation"] = database_location
        if logs_location is not UNSET:
            field_dict["logsLocation"] = logs_location
        if available_processors is not UNSET:
            field_dict["availableProcessors"] = available_processors
        if available_memory is not UNSET:
            field_dict["availableMemory"] = available_memory
        if allocated_memory is not UNSET:
            field_dict["allocatedMemory"] = allocated_memory
        if used_memory is not UNSET:
            field_dict["usedMemory"] = used_memory
        if uptime is not UNSET:
            field_dict["uptime"] = uptime
        if started_time is not UNSET:
            field_dict["startedTime"] = started_time
        if database_background_threads is not UNSET:
            field_dict["databaseBackgroundThreads"] = database_background_threads
        if pending_async_jobs is not UNSET:
            field_dict["pendingAsyncJobs"] = pending_async_jobs
        if cached_results_count_in_db_queries_cache is not UNSET:
            field_dict["cachedResultsCountInDBQueriesCache"] = cached_results_count_in_db_queries_cache
        if database_queries_cache_hit_rate is not UNSET:
            field_dict["databaseQueriesCacheHitRate"] = database_queries_cache_hit_rate
        if blob_strings_cache_hit_rate is not UNSET:
            field_dict["blobStringsCacheHitRate"] = blob_strings_cache_hit_rate
        if total_transactions is not UNSET:
            field_dict["totalTransactions"] = total_transactions
        if transactions_per_second is not UNSET:
            field_dict["transactionsPerSecond"] = transactions_per_second
        if requests_per_second is not UNSET:
            field_dict["requestsPerSecond"] = requests_per_second
        if database_size is not UNSET:
            field_dict["databaseSize"] = database_size
        if full_database_size is not UNSET:
            field_dict["fullDatabaseSize"] = full_database_size
        if text_index_size is not UNSET:
            field_dict["textIndexSize"] = text_index_size
        if online_users is not UNSET:
            field_dict["onlineUsers"] = online_users
        if report_calculator_threads is not UNSET:
            field_dict["reportCalculatorThreads"] = report_calculator_threads
        if notification_analyzer_threads is not UNSET:
            field_dict["notificationAnalyzerThreads"] = notification_analyzer_threads
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import online_users as online_users_m
        except ImportError:
            import sys

            online_users_m = sys.modules[__package__ + "online_users"]

        d = src_dict.copy()

        installation_folder = d.pop("installationFolder", UNSET)

        database_location = d.pop("databaseLocation", UNSET)

        logs_location = d.pop("logsLocation", UNSET)

        available_processors = d.pop("availableProcessors", UNSET)

        available_memory = d.pop("availableMemory", UNSET)

        allocated_memory = d.pop("allocatedMemory", UNSET)

        used_memory = d.pop("usedMemory", UNSET)

        uptime = d.pop("uptime", UNSET)

        started_time = d.pop("startedTime", UNSET)

        database_background_threads = d.pop("databaseBackgroundThreads", UNSET)

        pending_async_jobs = d.pop("pendingAsyncJobs", UNSET)

        cached_results_count_in_db_queries_cache = d.pop("cachedResultsCountInDBQueriesCache", UNSET)

        database_queries_cache_hit_rate = d.pop("databaseQueriesCacheHitRate", UNSET)

        blob_strings_cache_hit_rate = d.pop("blobStringsCacheHitRate", UNSET)

        total_transactions = d.pop("totalTransactions", UNSET)

        transactions_per_second = d.pop("transactionsPerSecond", UNSET)

        requests_per_second = d.pop("requestsPerSecond", UNSET)

        database_size = d.pop("databaseSize", UNSET)

        full_database_size = d.pop("fullDatabaseSize", UNSET)

        text_index_size = d.pop("textIndexSize", UNSET)

        _online_users = d.pop("onlineUsers", UNSET)
        online_users: Union[Unset, online_users_m.OnlineUsers]
        if isinstance(_online_users, Unset):
            online_users = UNSET
        else:
            online_users = online_users_m.OnlineUsers.from_dict(_online_users)

        report_calculator_threads = d.pop("reportCalculatorThreads", UNSET)

        notification_analyzer_threads = d.pop("notificationAnalyzerThreads", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        telemetry = cls(
            installation_folder=installation_folder,
            database_location=database_location,
            logs_location=logs_location,
            available_processors=available_processors,
            available_memory=available_memory,
            allocated_memory=allocated_memory,
            used_memory=used_memory,
            uptime=uptime,
            started_time=started_time,
            database_background_threads=database_background_threads,
            pending_async_jobs=pending_async_jobs,
            cached_results_count_in_db_queries_cache=cached_results_count_in_db_queries_cache,
            database_queries_cache_hit_rate=database_queries_cache_hit_rate,
            blob_strings_cache_hit_rate=blob_strings_cache_hit_rate,
            total_transactions=total_transactions,
            transactions_per_second=transactions_per_second,
            requests_per_second=requests_per_second,
            database_size=database_size,
            full_database_size=full_database_size,
            text_index_size=text_index_size,
            online_users=online_users,
            report_calculator_threads=report_calculator_threads,
            notification_analyzer_threads=notification_analyzer_threads,
            id=id,
            type=type,
        )

        return telemetry



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BundleElement")


@attr.s(auto_attribs=True)
class BundleElement:
    """Represents a field value in YouTrack."""

    name: "Union[Unset, str]" = UNSET
    bundle: "Union[Unset, bundle_m.Bundle]" = UNSET
    description: "Union[Unset, str]" = UNSET
    ordinal: "Union[Unset, int]" = UNSET
    color: "Union[Unset, field_style_m.FieldStyle]" = UNSET
    has_running_job: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        bundle: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bundle, Unset):
            bundle = self.bundle.to_dict()

        description = self.description
        ordinal = self.ordinal
        color: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.color, Unset):
            color = self.color.to_dict()

        has_running_job = self.has_running_job
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if bundle is not UNSET:
            field_dict["bundle"] = bundle
        if description is not UNSET:
            field_dict["description"] = description
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if color is not UNSET:
            field_dict["color"] = color
        if has_running_job is not UNSET:
            field_dict["hasRunningJob"] = has_running_job
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import bundle as bundle_m
            from ..models import field_style as field_style_m
        except ImportError:
            import sys

            bundle_m = sys.modules[__package__ + "bundle"]
            field_style_m = sys.modules[__package__ + "field_style"]

        d = src_dict.copy()

        name = d.pop("name", UNSET)

        _bundle = d.pop("bundle", UNSET)
        bundle: Union[Unset, bundle_m.Bundle]
        if isinstance(_bundle, Unset):
            bundle = UNSET
        else:
            bundle = bundle_m.Bundle.from_dict(_bundle)

        description = d.pop("description", UNSET)

        ordinal = d.pop("ordinal", UNSET)

        _color = d.pop("color", UNSET)
        color: Union[Unset, field_style_m.FieldStyle]
        if isinstance(_color, Unset):
            color = UNSET
        else:
            color = field_style_m.FieldStyle.from_dict(_color)

        has_running_job = d.pop("hasRunningJob", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        bundle_element = cls(
            name=name,
            bundle=bundle,
            description=description,
            ordinal=ordinal,
            color=color,
            has_running_job=has_running_job,
            id=id,
            type=type,
        )

        return bundle_element



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SimpleIssueCustomField")


try:
    from ..models import issue_custom_field
except ImportError:
    import sys

    issue_custom_field = sys.modules[__package__ + "issue_custom_field"]


@attr.s(auto_attribs=True)
class SimpleIssueCustomField(issue_custom_field.IssueCustomField):
    """Represents the field of simple type in the issue."""

    project_custom_field: "Union[Unset, project_custom_field_m.ProjectCustomField]" = UNSET
    value: "Union[Unset, simple_issue_custom_field_value_m.SimpleIssueCustomFieldValue]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        project_custom_field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project_custom_field, Unset):
            project_custom_field = self.project_custom_field.to_dict()

        value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.value, Unset):
            value = self.value.to_dict()

        field_dict: Dict[str, Any] = {}
        _IssueCustomField_dict = super().to_dict()
        field_dict.update(_IssueCustomField_dict)
        field_dict.update({})
        if project_custom_field is not UNSET:
            field_dict["projectCustomField"] = project_custom_field
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import project_custom_field as project_custom_field_m
            from ..models import simple_issue_custom_field_value as simple_issue_custom_field_value_m
        except ImportError:
            import sys

            simple_issue_custom_field_value_m = sys.modules[__package__ + "simple_issue_custom_field_value"]
            project_custom_field_m = sys.modules[__package__ + "project_custom_field"]

        d = src_dict.copy()

        _IssueCustomField_kwargs = super().from_dict(src_dict=d).to_dict()
        _IssueCustomField_kwargs.pop("$type")

        _project_custom_field = d.pop("projectCustomField", UNSET)
        project_custom_field: Union[Unset, project_custom_field_m.ProjectCustomField]
        if isinstance(_project_custom_field, Unset):
            project_custom_field = UNSET
        else:
            project_custom_field = project_custom_field_m.ProjectCustomField.from_dict(_project_custom_field)

        _value = d.pop("value", UNSET)
        value: Union[Unset, simple_issue_custom_field_value_m.SimpleIssueCustomFieldValue]
        if isinstance(_value, Unset):
            value = UNSET
        else:
            value = simple_issue_custom_field_value_m.SimpleIssueCustomFieldValue.from_dict(_value)

        simple_issue_custom_field = cls(
            project_custom_field=project_custom_field,
            value=value,
            **_IssueCustomField_kwargs,
        )

        return simple_issue_custom_field



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SearchSuggestions")


@attr.s(auto_attribs=True)
class SearchSuggestions:
    """Represents the list of search suggestions for the currently entered search query."""

    caret: "Union[Unset, int]" = UNSET
    ignore_unresolved_setting: "Union[Unset, bool]" = UNSET
    query: "Union[Unset, str]" = UNSET
    suggestions: "Union[Unset, List[suggestion_m.Suggestion]]" = UNSET
    folders: "Union[Unset, List[issue_folder_m.IssueFolder]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        caret = self.caret
        ignore_unresolved_setting = self.ignore_unresolved_setting
        query = self.query
        suggestions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.suggestions, Unset):
            suggestions = []
            for suggestions_item_data in self.suggestions:
                suggestions_item = suggestions_item_data.to_dict()

                suggestions.append(suggestions_item)

        folders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.folders, Unset):
            folders = []
            for folders_item_data in self.folders:
                folders_item = folders_item_data.to_dict()

                folders.append(folders_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if caret is not UNSET:
            field_dict["caret"] = caret
        if ignore_unresolved_setting is not UNSET:
            field_dict["ignoreUnresolvedSetting"] = ignore_unresolved_setting
        if query is not UNSET:
            field_dict["query"] = query
        if suggestions is not UNSET:
            field_dict["suggestions"] = suggestions
        if folders is not UNSET:
            field_dict["folders"] = folders
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue_folder as issue_folder_m
            from ..models import suggestion as suggestion_m
        except ImportError:
            import sys

            issue_folder_m = sys.modules[__package__ + "issue_folder"]
            suggestion_m = sys.modules[__package__ + "suggestion"]

        d = src_dict.copy()

        caret = d.pop("caret", UNSET)

        ignore_unresolved_setting = d.pop("ignoreUnresolvedSetting", UNSET)

        query = d.pop("query", UNSET)

        suggestions = []
        _suggestions = d.pop("suggestions", UNSET)
        for suggestions_item_data in _suggestions or []:
            suggestions_item = suggestion_m.Suggestion.from_dict(suggestions_item_data)

            suggestions.append(suggestions_item)

        folders = []
        _folders = d.pop("folders", UNSET)
        for folders_item_data in _folders or []:
            folders_item = issue_folder_m.IssueFolder.from_dict(folders_item_data)

            folders.append(folders_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        search_suggestions = cls(
            caret=caret,
            ignore_unresolved_setting=ignore_unresolved_setting,
            query=query,
            suggestions=suggestions,
            folders=folders,
            id=id,
            type=type,
        )

        return search_suggestions



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ParsedCommand")


@attr.s(auto_attribs=True)
class ParsedCommand:
    """Represents the command that was parsed from the provided query."""

    description: "Union[Unset, str]" = UNSET
    error: "Union[Unset, bool]" = UNSET
    delete: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        description = self.description
        error = self.error
        delete = self.delete
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if error is not UNSET:
            field_dict["error"] = error
        if delete is not UNSET:
            field_dict["delete"] = delete
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        description = d.pop("description", UNSET)

        error = d.pop("error", UNSET)

        delete = d.pop("delete", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        parsed_command = cls(
            description=description,
            error=error,
            delete=delete,
            id=id,
            type=type,
        )

        return parsed_command



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="PeriodProjectCustomField")


try:
    from ..models import project_custom_field
except ImportError:
    import sys

    project_custom_field = sys.modules[__package__ + "project_custom_field"]


@attr.s(auto_attribs=True)
class PeriodProjectCustomField(project_custom_field.ProjectCustomField):
    """Represents project settings for the period field."""

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        _ProjectCustomField_dict = super().to_dict()
        field_dict.update(_ProjectCustomField_dict)
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _ProjectCustomField_kwargs = super().from_dict(src_dict=d).to_dict()
        _ProjectCustomField_kwargs.pop("$type")

        period_project_custom_field = cls(
            **_ProjectCustomField_kwargs,
        )

        return period_project_custom_field



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BackupFile")


@attr.s(auto_attribs=True)
class BackupFile:
    """Represents the backup file."""

    name: "Union[Unset, str]" = UNSET
    size: "Union[Unset, int]" = UNSET
    creation_date: "Union[Unset, int]" = UNSET
    link: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        size = self.size
        creation_date = self.creation_date
        link = self.link
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if size is not UNSET:
            field_dict["size"] = size
        if creation_date is not UNSET:
            field_dict["creationDate"] = creation_date
        if link is not UNSET:
            field_dict["link"] = link
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        name = d.pop("name", UNSET)

        size = d.pop("size", UNSET)

        creation_date = d.pop("creationDate", UNSET)

        link = d.pop("link", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        backup_file = cls(
            name=name,
            size=size,
            creation_date=creation_date,
            link=link,
            id=id,
            type=type,
        )

        return backup_file



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserProfiles")


@attr.s(auto_attribs=True)
class UserProfiles:
    """ """

    general: "Union[Unset, general_user_profile_m.GeneralUserProfile]" = UNSET
    notifications: "Union[Unset, notifications_user_profile_m.NotificationsUserProfile]" = UNSET
    timetracking: "Union[Unset, time_tracking_user_profile_m.TimeTrackingUserProfile]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        general: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.general, Unset):
            general = self.general.to_dict()

        notifications: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.notifications, Unset):
            notifications = self.notifications.to_dict()

        timetracking: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.timetracking, Unset):
            timetracking = self.timetracking.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if general is not UNSET:
            field_dict["general"] = general
        if notifications is not UNSET:
            field_dict["notifications"] = notifications
        if timetracking is not UNSET:
            field_dict["timetracking"] = timetracking
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import general_user_profile as general_user_profile_m
            from ..models import notifications_user_profile as notifications_user_profile_m
            from ..models import time_tracking_user_profile as time_tracking_user_profile_m
        except ImportError:
            import sys

            general_user_profile_m = sys.modules[__package__ + "general_user_profile"]
            time_tracking_user_profile_m = sys.modules[__package__ + "time_tracking_user_profile"]
            notifications_user_profile_m = sys.modules[__package__ + "notifications_user_profile"]

        d = src_dict.copy()

        _general = d.pop("general", UNSET)
        general: Union[Unset, general_user_profile_m.GeneralUserProfile]
        if isinstance(_general, Unset):
            general = UNSET
        else:
            general = general_user_profile_m.GeneralUserProfile.from_dict(_general)

        _notifications = d.pop("notifications", UNSET)
        notifications: Union[Unset, notifications_user_profile_m.NotificationsUserProfile]
        if isinstance(_notifications, Unset):
            notifications = UNSET
        else:
            notifications = notifications_user_profile_m.NotificationsUserProfile.from_dict(_notifications)

        _timetracking = d.pop("timetracking", UNSET)
        timetracking: Union[Unset, time_tracking_user_profile_m.TimeTrackingUserProfile]
        if isinstance(_timetracking, Unset):
            timetracking = UNSET
        else:
            timetracking = time_tracking_user_profile_m.TimeTrackingUserProfile.from_dict(_timetracking)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        user_profiles = cls(
            general=general,
            notifications=notifications,
            timetracking=timetracking,
            id=id,
            type=type,
        )

        return user_profiles



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GlobalTimeTrackingSettings")


@attr.s(auto_attribs=True)
class GlobalTimeTrackingSettings:
    """Represents time tracking settings of your server."""

    work_item_types: "Union[Unset, List[work_item_type_m.WorkItemType]]" = UNSET
    work_time_settings: "Union[Unset, work_time_settings_m.WorkTimeSettings]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        work_item_types: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.work_item_types, Unset):
            work_item_types = []
            for work_item_types_item_data in self.work_item_types:
                work_item_types_item = work_item_types_item_data.to_dict()

                work_item_types.append(work_item_types_item)

        work_time_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.work_time_settings, Unset):
            work_time_settings = self.work_time_settings.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if work_item_types is not UNSET:
            field_dict["workItemTypes"] = work_item_types
        if work_time_settings is not UNSET:
            field_dict["workTimeSettings"] = work_time_settings
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import work_item_type as work_item_type_m
            from ..models import work_time_settings as work_time_settings_m
        except ImportError:
            import sys

            work_time_settings_m = sys.modules[__package__ + "work_time_settings"]
            work_item_type_m = sys.modules[__package__ + "work_item_type"]

        d = src_dict.copy()

        work_item_types = []
        _work_item_types = d.pop("workItemTypes", UNSET)
        for work_item_types_item_data in _work_item_types or []:
            work_item_types_item = work_item_type_m.WorkItemType.from_dict(work_item_types_item_data)

            work_item_types.append(work_item_types_item)

        _work_time_settings = d.pop("workTimeSettings", UNSET)
        work_time_settings: Union[Unset, work_time_settings_m.WorkTimeSettings]
        if isinstance(_work_time_settings, Unset):
            work_time_settings = UNSET
        else:
            work_time_settings = work_time_settings_m.WorkTimeSettings.from_dict(_work_time_settings)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        global_time_tracking_settings = cls(
            work_item_types=work_item_types,
            work_time_settings=work_time_settings,
            id=id,
            type=type,
        )

        return global_time_tracking_settings



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueTimeTracker")


@attr.s(auto_attribs=True)
class IssueTimeTracker:
    """Represents time tracking settings in the issue."""

    work_items: "Union[Unset, List[issue_work_item_m.IssueWorkItem]]" = UNSET
    enabled: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        work_items: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.work_items, Unset):
            work_items = []
            for work_items_item_data in self.work_items:
                work_items_item = work_items_item_data.to_dict()

                work_items.append(work_items_item)

        enabled = self.enabled
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if work_items is not UNSET:
            field_dict["workItems"] = work_items
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue_work_item as issue_work_item_m
        except ImportError:
            import sys

            issue_work_item_m = sys.modules[__package__ + "issue_work_item"]

        d = src_dict.copy()

        work_items = []
        _work_items = d.pop("workItems", UNSET)
        for work_items_item_data in _work_items or []:
            work_items_item = issue_work_item_m.IssueWorkItem.from_dict(work_items_item_data)

            work_items.append(work_items_item)

        enabled = d.pop("enabled", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue_time_tracker = cls(
            work_items=work_items,
            enabled=enabled,
            id=id,
            type=type,
        )

        return issue_time_tracker



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DurationValue")


@attr.s(auto_attribs=True)
class DurationValue:
    """Represents the duration value and its visual presentation."""

    minutes: "Union[Unset, int]" = UNSET
    presentation: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        minutes = self.minutes
        presentation = self.presentation
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if minutes is not UNSET:
            field_dict["minutes"] = minutes
        if presentation is not UNSET:
            field_dict["presentation"] = presentation
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        minutes = d.pop("minutes", UNSET)

        presentation = d.pop("presentation", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        duration_value = cls(
            minutes=minutes,
            presentation=presentation,
            id=id,
            type=type,
        )

        return duration_value



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SprintsSettings")


@attr.s(auto_attribs=True)
class SprintsSettings:
    """Describes sprints configuration."""

    is_explicit: "Union[Unset, bool]" = UNSET
    card_on_several_sprints: "Union[Unset, bool]" = UNSET
    default_sprint: "Union[Unset, sprint_m.Sprint]" = UNSET
    disable_sprints: "Union[Unset, bool]" = UNSET
    explicit_query: "Union[Unset, str]" = UNSET
    sprint_sync_field: "Union[Unset, custom_field_m.CustomField]" = UNSET
    hide_subtasks_of_cards: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_explicit = self.is_explicit
        card_on_several_sprints = self.card_on_several_sprints
        default_sprint: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_sprint, Unset):
            default_sprint = self.default_sprint.to_dict()

        disable_sprints = self.disable_sprints
        explicit_query = self.explicit_query
        sprint_sync_field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sprint_sync_field, Unset):
            sprint_sync_field = self.sprint_sync_field.to_dict()

        hide_subtasks_of_cards = self.hide_subtasks_of_cards
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_explicit is not UNSET:
            field_dict["isExplicit"] = is_explicit
        if card_on_several_sprints is not UNSET:
            field_dict["cardOnSeveralSprints"] = card_on_several_sprints
        if default_sprint is not UNSET:
            field_dict["defaultSprint"] = default_sprint
        if disable_sprints is not UNSET:
            field_dict["disableSprints"] = disable_sprints
        if explicit_query is not UNSET:
            field_dict["explicitQuery"] = explicit_query
        if sprint_sync_field is not UNSET:
            field_dict["sprintSyncField"] = sprint_sync_field
        if hide_subtasks_of_cards is not UNSET:
            field_dict["hideSubtasksOfCards"] = hide_subtasks_of_cards
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import custom_field as custom_field_m
            from ..models import sprint as sprint_m
        except ImportError:
            import sys

            custom_field_m = sys.modules[__package__ + "custom_field"]
            sprint_m = sys.modules[__package__ + "sprint"]

        d = src_dict.copy()

        is_explicit = d.pop("isExplicit", UNSET)

        card_on_several_sprints = d.pop("cardOnSeveralSprints", UNSET)

        _default_sprint = d.pop("defaultSprint", UNSET)
        default_sprint: Union[Unset, sprint_m.Sprint]
        if isinstance(_default_sprint, Unset):
            default_sprint = UNSET
        else:
            default_sprint = sprint_m.Sprint.from_dict(_default_sprint)

        disable_sprints = d.pop("disableSprints", UNSET)

        explicit_query = d.pop("explicitQuery", UNSET)

        _sprint_sync_field = d.pop("sprintSyncField", UNSET)
        sprint_sync_field: Union[Unset, custom_field_m.CustomField]
        if isinstance(_sprint_sync_field, Unset):
            sprint_sync_field = UNSET
        else:
            sprint_sync_field = custom_field_m.CustomField.from_dict(_sprint_sync_field)

        hide_subtasks_of_cards = d.pop("hideSubtasksOfCards", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        sprints_settings = cls(
            is_explicit=is_explicit,
            card_on_several_sprints=card_on_several_sprints,
            default_sprint=default_sprint,
            disable_sprints=disable_sprints,
            explicit_query=explicit_query,
            sprint_sync_field=sprint_sync_field,
            hide_subtasks_of_cards=hide_subtasks_of_cards,
            id=id,
            type=type,
        )

        return sprints_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VersionBundleElement")


try:
    from ..models import bundle_element
except ImportError:
    import sys

    bundle_element = sys.modules[__package__ + "bundle_element"]


@attr.s(auto_attribs=True)
class VersionBundleElement(bundle_element.BundleElement):
    """Represents a version in YouTrack."""

    archived: "Union[Unset, bool]" = UNSET
    release_date: "Union[Unset, int]" = UNSET
    released: "Union[Unset, bool]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        archived = self.archived
        release_date = self.release_date
        released = self.released

        field_dict: Dict[str, Any] = {}
        _BundleElement_dict = super().to_dict()
        field_dict.update(_BundleElement_dict)
        field_dict.update({})
        if archived is not UNSET:
            field_dict["archived"] = archived
        if release_date is not UNSET:
            field_dict["releaseDate"] = release_date
        if released is not UNSET:
            field_dict["released"] = released

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _BundleElement_kwargs = super().from_dict(src_dict=d).to_dict()
        _BundleElement_kwargs.pop("$type")

        archived = d.pop("archived", UNSET)

        release_date = d.pop("releaseDate", UNSET)

        released = d.pop("released", UNSET)

        version_bundle_element = cls(
            archived=archived,
            release_date=release_date,
            released=released,
            **_BundleElement_kwargs,
        )

        return version_bundle_element



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AgileColumnFieldValue")


try:
    from ..models import database_attribute_value
except ImportError:
    import sys

    database_attribute_value = sys.modules[__package__ + "database_attribute_value"]


@attr.s(auto_attribs=True)
class AgileColumnFieldValue(database_attribute_value.DatabaseAttributeValue):
    """Represents a field value or values, parameterizing agile column."""

    name: "Union[Unset, str]" = UNSET
    is_resolved: "Union[Unset, bool]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        is_resolved = self.is_resolved

        field_dict: Dict[str, Any] = {}
        _DatabaseAttributeValue_dict = super().to_dict()
        field_dict.update(_DatabaseAttributeValue_dict)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if is_resolved is not UNSET:
            field_dict["isResolved"] = is_resolved

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _DatabaseAttributeValue_kwargs = super().from_dict(src_dict=d).to_dict()
        _DatabaseAttributeValue_kwargs.pop("$type")

        name = d.pop("name", UNSET)

        is_resolved = d.pop("isResolved", UNSET)

        agile_column_field_value = cls(
            name=name,
            is_resolved=is_resolved,
            **_DatabaseAttributeValue_kwargs,
        )

        return agile_column_field_value



import datetime
from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models import issue_work_item as issue_work_item_m
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    query: "Union[Unset, None, str]" = UNSET,
    start_date: "Union[Unset, None, datetime.date]" = UNSET,
    end_date: "Union[Unset, None, datetime.date]" = UNSET,
    start: "Union[Unset, None, int]" = UNSET,
    end: "Union[Unset, None, int]" = UNSET,
    created_start: "Union[Unset, None, int]" = UNSET,
    created_end: "Union[Unset, None, int]" = UNSET,
    updated_start: "Union[Unset, None, int]" = UNSET,
    updated_end: "Union[Unset, None, int]" = UNSET,
    author: "Union[Unset, None, str]" = UNSET,
    creator: "Union[Unset, None, str]" = UNSET,
    fields: "Union[Unset, None, str]" = "$type,author($type,id,login,name,ringId),created,creator($type,id,login,name,ringId),date,duration($type,id),id,text,updated,usesMarkdown",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Dict[str, Any]:
    url = "{}/workItems".format(client.youtrack_base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_start_date: Union[Unset, None, str] = UNSET
    if not isinstance(start_date, Unset):
        json_start_date = start_date.isoformat() if start_date else None

    json_end_date: Union[Unset, None, str] = UNSET
    if not isinstance(end_date, Unset):
        json_end_date = end_date.isoformat() if end_date else None

    params: Dict[str, Any] = {
        "query": query,
        "startDate": json_start_date,
        "endDate": json_end_date,
        "start": start,
        "end": end,
        "createdStart": created_start,
        "createdEnd": created_end,
        "updatedStart": updated_start,
        "updatedEnd": updated_end,
        "author": author,
        "creator": creator,
        "fields": fields,
        "$skip": skip,
        "$top": top,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[issue_work_item_m.IssueWorkItem]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = issue_work_item_m.IssueWorkItem.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[issue_work_item_m.IssueWorkItem]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    query: "Union[Unset, None, str]" = UNSET,
    start_date: "Union[Unset, None, datetime.date]" = UNSET,
    end_date: "Union[Unset, None, datetime.date]" = UNSET,
    start: "Union[Unset, None, int]" = UNSET,
    end: "Union[Unset, None, int]" = UNSET,
    created_start: "Union[Unset, None, int]" = UNSET,
    created_end: "Union[Unset, None, int]" = UNSET,
    updated_start: "Union[Unset, None, int]" = UNSET,
    updated_end: "Union[Unset, None, int]" = UNSET,
    author: "Union[Unset, None, str]" = UNSET,
    creator: "Union[Unset, None, str]" = UNSET,
    fields: "Union[Unset, None, str]" = "$type,author($type,id,login,name,ringId),created,creator($type,id,login,name,ringId),date,duration($type,id),id,text,updated,usesMarkdown",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Response[List[issue_work_item_m.IssueWorkItem]]:
    kwargs = _get_kwargs(
        client=client,
        query=query,
        start_date=start_date,
        end_date=end_date,
        start=start,
        end=end,
        created_start=created_start,
        created_end=created_end,
        updated_start=updated_start,
        updated_end=updated_end,
        author=author,
        creator=creator,
        fields=fields,
        skip=skip,
        top=top,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    query: "Union[Unset, None, str]" = UNSET,
    start_date: "Union[Unset, None, datetime.date]" = UNSET,
    end_date: "Union[Unset, None, datetime.date]" = UNSET,
    start: "Union[Unset, None, int]" = UNSET,
    end: "Union[Unset, None, int]" = UNSET,
    created_start: "Union[Unset, None, int]" = UNSET,
    created_end: "Union[Unset, None, int]" = UNSET,
    updated_start: "Union[Unset, None, int]" = UNSET,
    updated_end: "Union[Unset, None, int]" = UNSET,
    author: "Union[Unset, None, str]" = UNSET,
    creator: "Union[Unset, None, str]" = UNSET,
    fields: "Union[Unset, None, str]" = "$type,author($type,id,login,name,ringId),created,creator($type,id,login,name,ringId),date,duration($type,id),id,text,updated,usesMarkdown",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Optional[List[issue_work_item_m.IssueWorkItem]]:
    """ """

    return sync_detailed(
        client=client,
        query=query,
        start_date=start_date,
        end_date=end_date,
        start=start,
        end=end,
        created_start=created_start,
        created_end=created_end,
        updated_start=updated_start,
        updated_end=updated_end,
        author=author,
        creator=creator,
        fields=fields,
        skip=skip,
        top=top,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    query: "Union[Unset, None, str]" = UNSET,
    start_date: "Union[Unset, None, datetime.date]" = UNSET,
    end_date: "Union[Unset, None, datetime.date]" = UNSET,
    start: "Union[Unset, None, int]" = UNSET,
    end: "Union[Unset, None, int]" = UNSET,
    created_start: "Union[Unset, None, int]" = UNSET,
    created_end: "Union[Unset, None, int]" = UNSET,
    updated_start: "Union[Unset, None, int]" = UNSET,
    updated_end: "Union[Unset, None, int]" = UNSET,
    author: "Union[Unset, None, str]" = UNSET,
    creator: "Union[Unset, None, str]" = UNSET,
    fields: "Union[Unset, None, str]" = "$type,author($type,id,login,name,ringId),created,creator($type,id,login,name,ringId),date,duration($type,id),id,text,updated,usesMarkdown",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Response[List[issue_work_item_m.IssueWorkItem]]:
    kwargs = _get_kwargs(
        client=client,
        query=query,
        start_date=start_date,
        end_date=end_date,
        start=start,
        end=end,
        created_start=created_start,
        created_end=created_end,
        updated_start=updated_start,
        updated_end=updated_end,
        author=author,
        creator=creator,
        fields=fields,
        skip=skip,
        top=top,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    query: "Union[Unset, None, str]" = UNSET,
    start_date: "Union[Unset, None, datetime.date]" = UNSET,
    end_date: "Union[Unset, None, datetime.date]" = UNSET,
    start: "Union[Unset, None, int]" = UNSET,
    end: "Union[Unset, None, int]" = UNSET,
    created_start: "Union[Unset, None, int]" = UNSET,
    created_end: "Union[Unset, None, int]" = UNSET,
    updated_start: "Union[Unset, None, int]" = UNSET,
    updated_end: "Union[Unset, None, int]" = UNSET,
    author: "Union[Unset, None, str]" = UNSET,
    creator: "Union[Unset, None, str]" = UNSET,
    fields: "Union[Unset, None, str]" = "$type,author($type,id,login,name,ringId),created,creator($type,id,login,name,ringId),date,duration($type,id),id,text,updated,usesMarkdown",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Optional[List[issue_work_item_m.IssueWorkItem]]:
    """ """

    return (
        await asyncio_detailed(
            client=client,
            query=query,
            start_date=start_date,
            end_date=end_date,
            start=start,
            end=end,
            created_start=created_start,
            created_end=created_end,
            updated_start=updated_start,
            updated_end=updated_end,
            author=author,
            creator=creator,
            fields=fields,
            skip=skip,
            top=top,
        )
    ).parsed



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['youtrack_rest_client',
 'youtrack_rest_client.api',
 'youtrack_rest_client.api.default',
 'youtrack_rest_client.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<22.0.0',
 'httpx>=0.15.4,<0.19.0',
 'hub-rest-client>=0.1.5',
 'python-dateutil>=2.8.0,<3.0.0']

setup_kwargs = {
    'name': 'youtrack-rest-client',
    'version': '0.1.5',
    'description': 'A client library for accessing YouTrack REST API',
    'long_description': '# youtrack-rest-client `0.1.5`\nA client library for accessing YouTrack REST API\n\n⚠️This SDK generated using `2021.3` [OpenAPI](https://youtrack.jetbrains.com/api/openapi.json) version by custom [openapi-python-client](https://github.com/openapi-generators/openapi-python-client) which is under development.\n It may have some bugs. Use with caution.⚠️\n\nIf you find a bug or want to request a new feature, please create an issue in [YouTrack](https://youtrack.jetbrains.com/newIssue?project=JT&c=State%20Open&c=Subsystem%20Python%20client%20library).\n\n## Usage\nFirst, create a client:\n\n```python\nfrom youtrack_rest_client import Client\n\nclient = Client(\n    hub_base_url="https://hub.jetbrains.com/api/rest",\n    youtrack_base_url="https://youtrack.jetbrains.com/api"\n)\n```\n\nIf the endpoints you\'re going to hit require authentication, use `AuthenticatedClient` instead:\n\n```python\nfrom youtrack_rest_client import AuthenticatedClient\n\nclient = AuthenticatedClient(\n    hub_base_url="https://hub.jetbrains.com/api/rest",\n    youtrack_base_url="https://youtrack.jetbrains.com/api",\n    token=os.getenv("YOUTRACK_TOKEN")\n)\n```\n\nNow call your endpoint and use your models:\n\n```python\nfrom youtrack_rest_client.models import MyDataModel\nfrom youtrack_rest_client.api.my_tag import get_my_data_model\nfrom youtrack_rest_client.types import Response\n\nmy_data: MyDataModel = get_my_data_model.sync(client=client)\n# or if you need more info (e.g. status_code)\nresponse: Response[MyDataModel] = get_my_data_model.sync_detailed(client=client)\n```\n\nOr do the same thing with an async version:\n\n```python\nfrom youtrack_rest_client.models import MyDataModel\nfrom youtrack_rest_client.api.my_tag import get_my_data_model\nfrom youtrack_rest_client.types import Response\n\nmy_data: MyDataModel = await get_my_data_model.asyncio(client=client)\nresponse: Response[MyDataModel] = await get_my_data_model.asyncio_detailed(client=client)\n```\n\nThings to know:\n1. Every path/method combo becomes a Python module with four functions:\n    1. `sync`: Blocking request that returns parsed data (if successful) or `None`\n    1. `sync_detailed`: Blocking request that always returns a `Request`, optionally with `parsed` set if the request was successful.\n    1. `asyncio`: Like `sync` but the async instead of blocking\n    1. `asyncio_detailed`: Like `sync_detailed` by async instead of blocking\n1. All path/query params, and bodies become method arguments.\n1. If your endpoint had any tags on it, the first tag will be used as a module name for the function (my_tag above)\n1. Any endpoint which did not have a tag will be in `youtrack_rest_client.api.default`',
    'author': 'Matvey Ovtsin',
    'author_email': 'matvey.ovtsin@jetbrains.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SavedQuery")


try:
    from ..models import watch_folder
except ImportError:
    import sys

    watch_folder = sys.modules[__package__ + "watch_folder"]


@attr.s(auto_attribs=True)
class SavedQuery(watch_folder.WatchFolder):
    """Represents a saved search."""

    query: "Union[Unset, str]" = UNSET
    issues: "Union[Unset, List[issue_m.Issue]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        query = self.query
        issues: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.issues, Unset):
            issues = []
            for issues_item_data in self.issues:
                issues_item = issues_item_data.to_dict()

                issues.append(issues_item)

        field_dict: Dict[str, Any] = {}
        _WatchFolder_dict = super().to_dict()
        field_dict.update(_WatchFolder_dict)
        field_dict.update({})
        if query is not UNSET:
            field_dict["query"] = query
        if issues is not UNSET:
            field_dict["issues"] = issues

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]

        d = src_dict.copy()

        _WatchFolder_kwargs = super().from_dict(src_dict=d).to_dict()
        _WatchFolder_kwargs.pop("$type")

        query = d.pop("query", UNSET)

        issues = []
        _issues = d.pop("issues", UNSET)
        for issues_item_data in _issues or []:
            issues_item = issue_m.Issue.from_dict(issues_item_data)

            issues.append(issues_item)

        saved_query = cls(
            query=query,
            issues=issues,
            **_WatchFolder_kwargs,
        )

        return saved_query



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VisibilityActivityItem")


try:
    from ..models import multi_value_activity_item
except ImportError:
    import sys

    multi_value_activity_item = sys.modules[__package__ + "multi_value_activity_item"]


@attr.s(auto_attribs=True)
class VisibilityActivityItem(multi_value_activity_item.MultiValueActivityItem):
    """Represents the changes of properties responsible for visibility restriction.
    Can be <a href="api-entity-VisibilityGroupActivityItem.xml">VisibilityGroupActivityItem</a> or <a href="api-entity-VisibilityUserActivityItem.xml">VisibilityUserActivityItem</a>"""

    target_member: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        target_member = self.target_member

        field_dict: Dict[str, Any] = {}
        _MultiValueActivityItem_dict = super().to_dict()
        field_dict.update(_MultiValueActivityItem_dict)
        field_dict.update({})
        if target_member is not UNSET:
            field_dict["targetMember"] = target_member

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _MultiValueActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _MultiValueActivityItem_kwargs.pop("$type")

        target_member = d.pop("targetMember", UNSET)

        visibility_activity_item = cls(
            target_member=target_member,
            **_MultiValueActivityItem_kwargs,
        )

        return visibility_activity_item



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VisibilityUserActivityItem")


try:
    from ..models import visibility_activity_item
except ImportError:
    import sys

    visibility_activity_item = sys.modules[__package__ + "visibility_activity_item"]


@attr.s(auto_attribs=True)
class VisibilityUserActivityItem(visibility_activity_item.VisibilityActivityItem):
    """Represents the event when a user adds or removes a user to/from the Visibility settings of the target entity."""

    target_member: "Union[Unset, str]" = UNSET
    target_sub_member: "Union[Unset, str]" = UNSET
    removed: "Union[Unset, List[user_m.User]]" = UNSET
    added: "Union[Unset, List[user_m.User]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        target_member = self.target_member
        target_sub_member = self.target_sub_member
        removed: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.removed, Unset):
            removed = []
            for removed_item_data in self.removed:
                removed_item = removed_item_data.to_dict()

                removed.append(removed_item)

        added: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.added, Unset):
            added = []
            for added_item_data in self.added:
                added_item = added_item_data.to_dict()

                added.append(added_item)

        field_dict: Dict[str, Any] = {}
        _VisibilityActivityItem_dict = super().to_dict()
        field_dict.update(_VisibilityActivityItem_dict)
        field_dict.update({})
        if target_member is not UNSET:
            field_dict["targetMember"] = target_member
        if target_sub_member is not UNSET:
            field_dict["targetSubMember"] = target_sub_member
        if removed is not UNSET:
            field_dict["removed"] = removed
        if added is not UNSET:
            field_dict["added"] = added

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import user as user_m
        except ImportError:
            import sys

            user_m = sys.modules[__package__ + "user"]

        d = src_dict.copy()

        _VisibilityActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _VisibilityActivityItem_kwargs.pop("$type")

        target_member = d.pop("targetMember", UNSET)

        target_sub_member = d.pop("targetSubMember", UNSET)

        removed = []
        _removed = d.pop("removed", UNSET)
        for removed_item_data in _removed or []:
            removed_item = user_m.User.from_dict(removed_item_data)

            removed.append(removed_item)

        added = []
        _added = d.pop("added", UNSET)
        for added_item_data in _added or []:
            added_item = user_m.User.from_dict(added_item_data)

            added.append(added_item)

        visibility_user_activity_item = cls(
            target_member=target_member,
            target_sub_member=target_sub_member,
            removed=removed,
            added=added,
            **_VisibilityActivityItem_kwargs,
        )

        return visibility_user_activity_item



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomField")


@attr.s(auto_attribs=True)
class CustomField:
    """Represents a custom field in YouTrack."""

    name: "Union[Unset, str]" = UNSET
    localized_name: "Union[Unset, str]" = UNSET
    field_type: "Union[Unset, field_type_m.FieldType]" = UNSET
    is_auto_attached: "Union[Unset, bool]" = UNSET
    is_displayed_in_issue_list: "Union[Unset, bool]" = UNSET
    ordinal: "Union[Unset, int]" = UNSET
    aliases: "Union[Unset, str]" = UNSET
    field_defaults: "Union[Unset, custom_field_defaults_m.CustomFieldDefaults]" = UNSET
    has_running_job: "Union[Unset, bool]" = UNSET
    is_updateable: "Union[Unset, bool]" = UNSET
    instances: "Union[Unset, List[project_custom_field_m.ProjectCustomField]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        localized_name = self.localized_name
        field_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_type, Unset):
            field_type = self.field_type.to_dict()

        is_auto_attached = self.is_auto_attached
        is_displayed_in_issue_list = self.is_displayed_in_issue_list
        ordinal = self.ordinal
        aliases = self.aliases
        field_defaults: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_defaults, Unset):
            field_defaults = self.field_defaults.to_dict()

        has_running_job = self.has_running_job
        is_updateable = self.is_updateable
        instances: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.instances, Unset):
            instances = []
            for instances_item_data in self.instances:
                instances_item = instances_item_data.to_dict()

                instances.append(instances_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if localized_name is not UNSET:
            field_dict["localizedName"] = localized_name
        if field_type is not UNSET:
            field_dict["fieldType"] = field_type
        if is_auto_attached is not UNSET:
            field_dict["isAutoAttached"] = is_auto_attached
        if is_displayed_in_issue_list is not UNSET:
            field_dict["isDisplayedInIssueList"] = is_displayed_in_issue_list
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if aliases is not UNSET:
            field_dict["aliases"] = aliases
        if field_defaults is not UNSET:
            field_dict["fieldDefaults"] = field_defaults
        if has_running_job is not UNSET:
            field_dict["hasRunningJob"] = has_running_job
        if is_updateable is not UNSET:
            field_dict["isUpdateable"] = is_updateable
        if instances is not UNSET:
            field_dict["instances"] = instances
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import custom_field_defaults as custom_field_defaults_m
            from ..models import field_type as field_type_m
            from ..models import project_custom_field as project_custom_field_m
        except ImportError:
            import sys

            project_custom_field_m = sys.modules[__package__ + "project_custom_field"]
            field_type_m = sys.modules[__package__ + "field_type"]
            custom_field_defaults_m = sys.modules[__package__ + "custom_field_defaults"]

        d = src_dict.copy()

        name = d.pop("name", UNSET)

        localized_name = d.pop("localizedName", UNSET)

        _field_type = d.pop("fieldType", UNSET)
        field_type: Union[Unset, field_type_m.FieldType]
        if isinstance(_field_type, Unset):
            field_type = UNSET
        else:
            field_type = field_type_m.FieldType.from_dict(_field_type)

        is_auto_attached = d.pop("isAutoAttached", UNSET)

        is_displayed_in_issue_list = d.pop("isDisplayedInIssueList", UNSET)

        ordinal = d.pop("ordinal", UNSET)

        aliases = d.pop("aliases", UNSET)

        _field_defaults = d.pop("fieldDefaults", UNSET)
        field_defaults: Union[Unset, custom_field_defaults_m.CustomFieldDefaults]
        if isinstance(_field_defaults, Unset):
            field_defaults = UNSET
        else:
            field_defaults = custom_field_defaults_m.CustomFieldDefaults.from_dict(_field_defaults)

        has_running_job = d.pop("hasRunningJob", UNSET)

        is_updateable = d.pop("isUpdateable", UNSET)

        instances = []
        _instances = d.pop("instances", UNSET)
        for instances_item_data in _instances or []:
            instances_item = project_custom_field_m.ProjectCustomField.from_dict(instances_item_data)

            instances.append(instances_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        custom_field = cls(
            name=name,
            localized_name=localized_name,
            field_type=field_type,
            is_auto_attached=is_auto_attached,
            is_displayed_in_issue_list=is_displayed_in_issue_list,
            ordinal=ordinal,
            aliases=aliases,
            field_defaults=field_defaults,
            has_running_job=has_running_job,
            is_updateable=is_updateable,
            instances=instances,
            id=id,
            type=type,
        )

        return custom_field



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SprintActivityItem")


try:
    from ..models import multi_value_activity_item
except ImportError:
    import sys

    multi_value_activity_item = sys.modules[__package__ + "multi_value_activity_item"]


@attr.s(auto_attribs=True)
class SprintActivityItem(multi_value_activity_item.MultiValueActivityItem):
    """Represents a change in the issue when it was added to or removed from an agile board sprint."""

    target: "Union[Unset, issue_m.Issue]" = UNSET
    removed: "Union[Unset, List[sprint_m.Sprint]]" = UNSET
    added: "Union[Unset, List[sprint_m.Sprint]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        target: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.target, Unset):
            target = self.target.to_dict()

        removed: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.removed, Unset):
            removed = []
            for removed_item_data in self.removed:
                removed_item = removed_item_data.to_dict()

                removed.append(removed_item)

        added: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.added, Unset):
            added = []
            for added_item_data in self.added:
                added_item = added_item_data.to_dict()

                added.append(added_item)

        field_dict: Dict[str, Any] = {}
        _MultiValueActivityItem_dict = super().to_dict()
        field_dict.update(_MultiValueActivityItem_dict)
        field_dict.update({})
        if target is not UNSET:
            field_dict["target"] = target
        if removed is not UNSET:
            field_dict["removed"] = removed
        if added is not UNSET:
            field_dict["added"] = added

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
            from ..models import sprint as sprint_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            sprint_m = sys.modules[__package__ + "sprint"]

        d = src_dict.copy()

        _MultiValueActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _MultiValueActivityItem_kwargs.pop("$type")

        _target = d.pop("target", UNSET)
        target: Union[Unset, issue_m.Issue]
        if isinstance(_target, Unset):
            target = UNSET
        else:
            target = issue_m.Issue.from_dict(_target)

        removed = []
        _removed = d.pop("removed", UNSET)
        for removed_item_data in _removed or []:
            removed_item = sprint_m.Sprint.from_dict(removed_item_data)

            removed.append(removed_item)

        added = []
        _added = d.pop("added", UNSET)
        for added_item_data in _added or []:
            added_item = sprint_m.Sprint.from_dict(added_item_data)

            added.append(added_item)

        sprint_activity_item = cls(
            target=target,
            removed=removed,
            added=added,
            **_MultiValueActivityItem_kwargs,
        )

        return sprint_activity_item



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectActivityItem")


try:
    from ..models import single_value_activity_item
except ImportError:
    import sys

    single_value_activity_item = sys.modules[__package__ + "single_value_activity_item"]


@attr.s(auto_attribs=True)
class ProjectActivityItem(single_value_activity_item.SingleValueActivityItem):
    """Represents the change in the project attribute on an Issue."""

    target: "Union[Unset, issue_m.Issue]" = UNSET
    removed: "Union[Unset, project_activity_item_removed_m.ProjectActivityItemRemoved]" = UNSET
    added: "Union[Unset, project_activity_item_added_m.ProjectActivityItemAdded]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        target: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.target, Unset):
            target = self.target.to_dict()

        removed: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.removed, Unset):
            removed = self.removed.to_dict()

        added: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.added, Unset):
            added = self.added.to_dict()

        field_dict: Dict[str, Any] = {}
        _SingleValueActivityItem_dict = super().to_dict()
        field_dict.update(_SingleValueActivityItem_dict)
        field_dict.update({})
        if target is not UNSET:
            field_dict["target"] = target
        if removed is not UNSET:
            field_dict["removed"] = removed
        if added is not UNSET:
            field_dict["added"] = added

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
            from ..models import project_activity_item_added as project_activity_item_added_m
            from ..models import project_activity_item_removed as project_activity_item_removed_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            project_activity_item_added_m = sys.modules[__package__ + "project_activity_item_added"]
            project_activity_item_removed_m = sys.modules[__package__ + "project_activity_item_removed"]

        d = src_dict.copy()

        _SingleValueActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _SingleValueActivityItem_kwargs.pop("$type")

        _target = d.pop("target", UNSET)
        target: Union[Unset, issue_m.Issue]
        if isinstance(_target, Unset):
            target = UNSET
        else:
            target = issue_m.Issue.from_dict(_target)

        _removed = d.pop("removed", UNSET)
        removed: Union[Unset, project_activity_item_removed_m.ProjectActivityItemRemoved]
        if isinstance(_removed, Unset):
            removed = UNSET
        else:
            removed = project_activity_item_removed_m.ProjectActivityItemRemoved.from_dict(_removed)

        _added = d.pop("added", UNSET)
        added: Union[Unset, project_activity_item_added_m.ProjectActivityItemAdded]
        if isinstance(_added, Unset):
            added = UNSET
        else:
            added = project_activity_item_added_m.ProjectActivityItemAdded.from_dict(_added)

        project_activity_item = cls(
            target=target,
            removed=removed,
            added=added,
            **_SingleValueActivityItem_kwargs,
        )

        return project_activity_item



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DatabaseBackupSettings")


@attr.s(auto_attribs=True)
class DatabaseBackupSettings:
    """Represents database backup settings of the YouTrack instance."""

    location: "Union[Unset, str]" = UNSET
    files_to_keep: "Union[Unset, int]" = UNSET
    cron_expression: "Union[Unset, str]" = UNSET
    archive_format: "Union[Unset, DatabaseBackupSettingsArchiveFormat]" = UNSET
    is_on: "Union[Unset, bool]" = UNSET
    available_disk_space: "Union[Unset, int]" = UNSET
    notified_users: "Union[Unset, List[user_m.User]]" = UNSET
    backup_status: "Union[Unset, backup_status_m.BackupStatus]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        location = self.location
        files_to_keep = self.files_to_keep
        cron_expression = self.cron_expression
        archive_format: Union[Unset, str] = UNSET
        if not isinstance(self.archive_format, Unset):
            archive_format = self.archive_format.value

        is_on = self.is_on
        available_disk_space = self.available_disk_space
        notified_users: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.notified_users, Unset):
            notified_users = []
            for notified_users_item_data in self.notified_users:
                notified_users_item = notified_users_item_data.to_dict()

                notified_users.append(notified_users_item)

        backup_status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.backup_status, Unset):
            backup_status = self.backup_status.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if location is not UNSET:
            field_dict["location"] = location
        if files_to_keep is not UNSET:
            field_dict["filesToKeep"] = files_to_keep
        if cron_expression is not UNSET:
            field_dict["cronExpression"] = cron_expression
        if archive_format is not UNSET:
            field_dict["archiveFormat"] = archive_format
        if is_on is not UNSET:
            field_dict["isOn"] = is_on
        if available_disk_space is not UNSET:
            field_dict["availableDiskSpace"] = available_disk_space
        if notified_users is not UNSET:
            field_dict["notifiedUsers"] = notified_users
        if backup_status is not UNSET:
            field_dict["backupStatus"] = backup_status
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import backup_status as backup_status_m
            from ..models import user as user_m
            from ..models.database_backup_settings_archive_format import DatabaseBackupSettingsArchiveFormat
        except ImportError:
            import sys

            backup_status_m = sys.modules[__package__ + "backup_status"]
            user_m = sys.modules[__package__ + "user"]
            DatabaseBackupSettingsArchiveFormat = sys.modules[__package__ + "DatabaseBackupSettingsArchiveForm"]

        d = src_dict.copy()

        location = d.pop("location", UNSET)

        files_to_keep = d.pop("filesToKeep", UNSET)

        cron_expression = d.pop("cronExpression", UNSET)

        _archive_format = d.pop("archiveFormat", UNSET)
        archive_format: Union[Unset, DatabaseBackupSettingsArchiveFormat]
        if isinstance(_archive_format, Unset):
            archive_format = UNSET
        else:
            archive_format = DatabaseBackupSettingsArchiveFormat(_archive_format)

        is_on = d.pop("isOn", UNSET)

        available_disk_space = d.pop("availableDiskSpace", UNSET)

        notified_users = []
        _notified_users = d.pop("notifiedUsers", UNSET)
        for notified_users_item_data in _notified_users or []:
            notified_users_item = user_m.User.from_dict(notified_users_item_data)

            notified_users.append(notified_users_item)

        _backup_status = d.pop("backupStatus", UNSET)
        backup_status: Union[Unset, backup_status_m.BackupStatus]
        if isinstance(_backup_status, Unset):
            backup_status = UNSET
        else:
            backup_status = backup_status_m.BackupStatus.from_dict(_backup_status)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        database_backup_settings = cls(
            location=location,
            files_to_keep=files_to_keep,
            cron_expression=cron_expression,
            archive_format=archive_format,
            is_on=is_on,
            available_disk_space=available_disk_space,
            notified_users=notified_users,
            backup_status=backup_status,
            id=id,
            type=type,
        )

        return database_backup_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="JabberSettings")


@attr.s(auto_attribs=True)
class JabberSettings:
    """Represents jabber settings for this YouTrack installation."""

    is_enabled: "Union[Unset, bool]" = UNSET
    host: "Union[Unset, str]" = UNSET
    port: "Union[Unset, int]" = UNSET
    login: "Union[Unset, str]" = UNSET
    service_name: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_enabled = self.is_enabled
        host = self.host
        port = self.port
        login = self.login
        service_name = self.service_name
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_enabled is not UNSET:
            field_dict["isEnabled"] = is_enabled
        if host is not UNSET:
            field_dict["host"] = host
        if port is not UNSET:
            field_dict["port"] = port
        if login is not UNSET:
            field_dict["login"] = login
        if service_name is not UNSET:
            field_dict["serviceName"] = service_name
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        is_enabled = d.pop("isEnabled", UNSET)

        host = d.pop("host", UNSET)

        port = d.pop("port", UNSET)

        login = d.pop("login", UNSET)

        service_name = d.pop("serviceName", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        jabber_settings = cls(
            is_enabled=is_enabled,
            host=host,
            port=port,
            login=login,
            service_name=service_name,
            id=id,
            type=type,
        )

        return jabber_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FieldBasedColorCoding")


try:
    from ..models import color_coding
except ImportError:
    import sys

    color_coding = sys.modules[__package__ + "color_coding"]


@attr.s(auto_attribs=True)
class FieldBasedColorCoding(color_coding.ColorCoding):
    """Sets a card color based on a value of some custom field."""

    prototype: "Union[Unset, custom_field_m.CustomField]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        prototype: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.prototype, Unset):
            prototype = self.prototype.to_dict()

        field_dict: Dict[str, Any] = {}
        _ColorCoding_dict = super().to_dict()
        field_dict.update(_ColorCoding_dict)
        field_dict.update({})
        if prototype is not UNSET:
            field_dict["prototype"] = prototype

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import custom_field as custom_field_m
        except ImportError:
            import sys

            custom_field_m = sys.modules[__package__ + "custom_field"]

        d = src_dict.copy()

        _ColorCoding_kwargs = super().from_dict(src_dict=d).to_dict()
        _ColorCoding_kwargs.pop("$type")

        _prototype = d.pop("prototype", UNSET)
        prototype: Union[Unset, custom_field_m.CustomField]
        if isinstance(_prototype, Unset):
            prototype = UNSET
        else:
            prototype = custom_field_m.CustomField.from_dict(_prototype)

        field_based_color_coding = cls(
            prototype=prototype,
            **_ColorCoding_kwargs,
        )

        return field_based_color_coding



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserGroup")


@attr.s(auto_attribs=True)
class UserGroup:
    """Represents a group of users."""

    name: "Union[Unset, str]" = UNSET
    ring_id: "Union[Unset, str]" = UNSET
    users_count: "Union[Unset, int]" = UNSET
    icon: "Union[Unset, str]" = UNSET
    all_users_group: "Union[Unset, bool]" = UNSET
    team_for_project: "Union[Unset, project_m.Project]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        ring_id = self.ring_id
        users_count = self.users_count
        icon = self.icon
        all_users_group = self.all_users_group
        team_for_project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.team_for_project, Unset):
            team_for_project = self.team_for_project.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if ring_id is not UNSET:
            field_dict["ringId"] = ring_id
        if users_count is not UNSET:
            field_dict["usersCount"] = users_count
        if icon is not UNSET:
            field_dict["icon"] = icon
        if all_users_group is not UNSET:
            field_dict["allUsersGroup"] = all_users_group
        if team_for_project is not UNSET:
            field_dict["teamForProject"] = team_for_project
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import project as project_m
        except ImportError:
            import sys

            project_m = sys.modules[__package__ + "project"]

        d = src_dict.copy()

        name = d.pop("name", UNSET)

        ring_id = d.pop("ringId", UNSET)

        users_count = d.pop("usersCount", UNSET)

        icon = d.pop("icon", UNSET)

        all_users_group = d.pop("allUsersGroup", UNSET)

        _team_for_project = d.pop("teamForProject", UNSET)
        team_for_project: Union[Unset, project_m.Project]
        if isinstance(_team_for_project, Unset):
            team_for_project = UNSET
        else:
            team_for_project = project_m.Project.from_dict(_team_for_project)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        user_group = cls(
            name=name,
            ring_id=ring_id,
            users_count=users_count,
            icon=icon,
            all_users_group=all_users_group,
            team_for_project=team_for_project,
            id=id,
            type=type,
        )

        return user_group



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="UnlimitedVisibility")


try:
    from ..models import visibility
except ImportError:
    import sys

    visibility = sys.modules[__package__ + "visibility"]


@attr.s(auto_attribs=True)
class UnlimitedVisibility(visibility.Visibility):
    """Represents unlimited visibility."""

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        _Visibility_dict = super().to_dict()
        field_dict.update(_Visibility_dict)
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _Visibility_kwargs = super().from_dict(src_dict=d).to_dict()
        _Visibility_kwargs.pop("$type")

        unlimited_visibility = cls(
            **_Visibility_kwargs,
        )

        return unlimited_visibility



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="WatchFolder")


try:
    from ..models import issue_folder
except ImportError:
    import sys

    issue_folder = sys.modules[__package__ + "issue_folder"]


@attr.s(auto_attribs=True)
class WatchFolder(issue_folder.IssueFolder):
    """A `WatchFolder` is an `IssueFolder` that let you enable notifications for a set
    of issues that it enfolds. It is a common abstract ancestor for saved searches and issue tags."""

    owner: "Union[Unset, user_m.User]" = UNSET
    visible_for: "Union[Unset, user_group_m.UserGroup]" = UNSET
    updateable_by: "Union[Unset, user_group_m.UserGroup]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        visible_for: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.visible_for, Unset):
            visible_for = self.visible_for.to_dict()

        updateable_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.updateable_by, Unset):
            updateable_by = self.updateable_by.to_dict()

        field_dict: Dict[str, Any] = {}
        _IssueFolder_dict = super().to_dict()
        field_dict.update(_IssueFolder_dict)
        field_dict.update({})
        if owner is not UNSET:
            field_dict["owner"] = owner
        if visible_for is not UNSET:
            field_dict["visibleFor"] = visible_for
        if updateable_by is not UNSET:
            field_dict["updateableBy"] = updateable_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import user as user_m
            from ..models import user_group as user_group_m
        except ImportError:
            import sys

            user_group_m = sys.modules[__package__ + "user_group"]
            user_m = sys.modules[__package__ + "user"]

        d = src_dict.copy()

        _IssueFolder_kwargs = super().from_dict(src_dict=d).to_dict()
        _IssueFolder_kwargs.pop("$type")

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, user_m.User]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = user_m.User.from_dict(_owner)

        _visible_for = d.pop("visibleFor", UNSET)
        visible_for: Union[Unset, user_group_m.UserGroup]
        if isinstance(_visible_for, Unset):
            visible_for = UNSET
        else:
            visible_for = user_group_m.UserGroup.from_dict(_visible_for)

        _updateable_by = d.pop("updateableBy", UNSET)
        updateable_by: Union[Unset, user_group_m.UserGroup]
        if isinstance(_updateable_by, Unset):
            updateable_by = UNSET
        else:
            updateable_by = user_group_m.UserGroup.from_dict(_updateable_by)

        watch_folder = cls(
            owner=owner,
            visible_for=visible_for,
            updateable_by=updateable_by,
            **_IssueFolder_kwargs,
        )

        return watch_folder



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectCustomField")


@attr.s(auto_attribs=True)
class ProjectCustomField:
    """Represents custom field settings for the particular project."""

    field: "Union[Unset, custom_field_m.CustomField]" = UNSET
    project: "Union[Unset, project_m.Project]" = UNSET
    can_be_empty: "Union[Unset, bool]" = UNSET
    empty_field_text: "Union[Unset, str]" = UNSET
    ordinal: "Union[Unset, int]" = UNSET
    is_public: "Union[Unset, bool]" = UNSET
    has_running_job: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field, Unset):
            field = self.field.to_dict()

        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        can_be_empty = self.can_be_empty
        empty_field_text = self.empty_field_text
        ordinal = self.ordinal
        is_public = self.is_public
        has_running_job = self.has_running_job
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if field is not UNSET:
            field_dict["field"] = field
        if project is not UNSET:
            field_dict["project"] = project
        if can_be_empty is not UNSET:
            field_dict["canBeEmpty"] = can_be_empty
        if empty_field_text is not UNSET:
            field_dict["emptyFieldText"] = empty_field_text
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if is_public is not UNSET:
            field_dict["isPublic"] = is_public
        if has_running_job is not UNSET:
            field_dict["hasRunningJob"] = has_running_job
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import custom_field as custom_field_m
            from ..models import project as project_m
        except ImportError:
            import sys

            project_m = sys.modules[__package__ + "project"]
            custom_field_m = sys.modules[__package__ + "custom_field"]

        d = src_dict.copy()

        _field = d.pop("field", UNSET)
        field: Union[Unset, custom_field_m.CustomField]
        if isinstance(_field, Unset):
            field = UNSET
        else:
            field = custom_field_m.CustomField.from_dict(_field)

        _project = d.pop("project", UNSET)
        project: Union[Unset, project_m.Project]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = project_m.Project.from_dict(_project)

        can_be_empty = d.pop("canBeEmpty", UNSET)

        empty_field_text = d.pop("emptyFieldText", UNSET)

        ordinal = d.pop("ordinal", UNSET)

        is_public = d.pop("isPublic", UNSET)

        has_running_job = d.pop("hasRunningJob", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        project_custom_field = cls(
            field=field,
            project=project,
            can_be_empty=can_be_empty,
            empty_field_text=empty_field_text,
            ordinal=ordinal,
            is_public=is_public,
            has_running_job=has_running_job,
            id=id,
            type=type,
        )

        return project_custom_field



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectTimeTrackingSettings")


@attr.s(auto_attribs=True)
class ProjectTimeTrackingSettings:
    """Represents time tracking settings of the project."""

    enabled: "Union[Unset, bool]" = UNSET
    estimate: "Union[Unset, project_custom_field_m.ProjectCustomField]" = UNSET
    time_spent: "Union[Unset, project_custom_field_m.ProjectCustomField]" = UNSET
    work_item_types: "Union[Unset, List[work_item_type_m.WorkItemType]]" = UNSET
    project: "Union[Unset, project_m.Project]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        estimate: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.estimate, Unset):
            estimate = self.estimate.to_dict()

        time_spent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_spent, Unset):
            time_spent = self.time_spent.to_dict()

        work_item_types: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.work_item_types, Unset):
            work_item_types = []
            for work_item_types_item_data in self.work_item_types:
                work_item_types_item = work_item_types_item_data.to_dict()

                work_item_types.append(work_item_types_item)

        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if estimate is not UNSET:
            field_dict["estimate"] = estimate
        if time_spent is not UNSET:
            field_dict["timeSpent"] = time_spent
        if work_item_types is not UNSET:
            field_dict["workItemTypes"] = work_item_types
        if project is not UNSET:
            field_dict["project"] = project
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import project as project_m
            from ..models import project_custom_field as project_custom_field_m
            from ..models import work_item_type as work_item_type_m
        except ImportError:
            import sys

            project_m = sys.modules[__package__ + "project"]
            project_custom_field_m = sys.modules[__package__ + "project_custom_field"]
            work_item_type_m = sys.modules[__package__ + "work_item_type"]

        d = src_dict.copy()

        enabled = d.pop("enabled", UNSET)

        _estimate = d.pop("estimate", UNSET)
        estimate: Union[Unset, project_custom_field_m.ProjectCustomField]
        if isinstance(_estimate, Unset):
            estimate = UNSET
        else:
            estimate = project_custom_field_m.ProjectCustomField.from_dict(_estimate)

        _time_spent = d.pop("timeSpent", UNSET)
        time_spent: Union[Unset, project_custom_field_m.ProjectCustomField]
        if isinstance(_time_spent, Unset):
            time_spent = UNSET
        else:
            time_spent = project_custom_field_m.ProjectCustomField.from_dict(_time_spent)

        work_item_types = []
        _work_item_types = d.pop("workItemTypes", UNSET)
        for work_item_types_item_data in _work_item_types or []:
            work_item_types_item = work_item_type_m.WorkItemType.from_dict(work_item_types_item_data)

            work_item_types.append(work_item_types_item)

        _project = d.pop("project", UNSET)
        project: Union[Unset, project_m.Project]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = project_m.Project.from_dict(_project)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        project_time_tracking_settings = cls(
            enabled=enabled,
            estimate=estimate,
            time_spent=time_spent,
            work_item_types=work_item_types,
            project=project,
            id=id,
            type=type,
        )

        return project_time_tracking_settings



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ColumnSettings")


@attr.s(auto_attribs=True)
class ColumnSettings:
    """Agile board columns settings."""

    field: "Union[Unset, custom_field_m.CustomField]" = UNSET
    columns: "Union[Unset, List[agile_column_m.AgileColumn]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field, Unset):
            field = self.field.to_dict()

        columns: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.columns, Unset):
            columns = []
            for columns_item_data in self.columns:
                columns_item = columns_item_data.to_dict()

                columns.append(columns_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if field is not UNSET:
            field_dict["field"] = field
        if columns is not UNSET:
            field_dict["columns"] = columns
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import agile_column as agile_column_m
            from ..models import custom_field as custom_field_m
        except ImportError:
            import sys

            agile_column_m = sys.modules[__package__ + "agile_column"]
            custom_field_m = sys.modules[__package__ + "custom_field"]

        d = src_dict.copy()

        _field = d.pop("field", UNSET)
        field: Union[Unset, custom_field_m.CustomField]
        if isinstance(_field, Unset):
            field = UNSET
        else:
            field = custom_field_m.CustomField.from_dict(_field)

        columns = []
        _columns = d.pop("columns", UNSET)
        for columns_item_data in _columns or []:
            columns_item = agile_column_m.AgileColumn.from_dict(columns_item_data)

            columns.append(columns_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        column_settings = cls(
            field=field,
            columns=columns,
            id=id,
            type=type,
        )

        return column_settings
